# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import argparse
import io

from botocore.exceptions import ClientError
from botocore.model import Shape

from awscli.customizations import utils
from awscli.customizations.exceptions import ParamValidationError
from awscli.testutils import BaseAWSHelpOutputTest, mock, unittest


class FakeParsedArgs:
    def __init__(self, **kwargs):
        self.__dict__.update(kwargs)


class TestCommandTableRenames(BaseAWSHelpOutputTest):
    def test_rename_command_table(self):
        handler = lambda command_table, **kwargs: utils.rename_command(
            command_table, 'ec2', 'fooec2'
        )
        # Verify that we can rename a top level command.
        self.session.register('building-command-table.main', handler)
        self.driver.main(['fooec2', 'help'])
        self.assert_contains('fooec2')

        # We can also see subcommands help as well.
        self.driver.main(['fooec2', 'run-instances', 'help'])
        self.assert_contains('run-instances')


class TestCommandTableAlias(BaseAWSHelpOutputTest):
    def test_alias_command_table(self):
        old_name = 'cloudhsmv2'
        new_name = 'nopossiblewaythisisalreadythere'

        def handler(command_table, **kwargs):
            utils.alias_command(command_table, old_name, new_name)

        self._assert_command_exists(old_name, handler)
        self._assert_command_exists(new_name, handler)

        # Verify that the new name is documented
        self.driver.main(['help'])
        self.assert_contains(new_name)
        self.assert_not_contains(old_name)

    def _assert_command_exists(self, command_name, handler):
        # Verify that we can alias a top level command.
        self.session.register('building-command-table.main', handler)
        self.driver.main([command_name, 'help'])
        self.assert_contains(command_name)

        # We can also see subcommands help as well.
        self.driver.main([command_name, 'describe-clusters', 'help'])
        self.assert_contains('describe-clusters')


class TestHiddenAlias(unittest.TestCase):
    def test_not_marked_as_required_if_not_needed(self):
        original_arg_required = mock.Mock()
        original_arg_required.required = False
        arg_table = {'original': original_arg_required}
        utils.make_hidden_alias(arg_table, 'original', 'new-name')
        self.assertIn('new-name', arg_table)
        # Note: the _DOCUMENT_AS_REQUIRED is tested as a functional
        # test because it only affects how the doc synopsis is
        # rendered.
        self.assertFalse(arg_table['original'].required)
        self.assertFalse(arg_table['new-name'].required)

    def test_hidden_alias_marks_as_not_required(self):
        original_arg_required = mock.Mock()
        original_arg_required.required = True
        arg_table = {'original': original_arg_required}
        utils.make_hidden_alias(arg_table, 'original', 'new-name')
        self.assertIn('new-name', arg_table)
        self.assertFalse(arg_table['original'].required)
        self.assertFalse(arg_table['new-name'].required)


class TestValidateMututuallyExclusiveGroups(unittest.TestCase):
    def test_two_single_groups(self):
        # The most basic example of mutually exclusive args.
        # If foo is specified, but bar is not, then we're fine.
        parsed = FakeParsedArgs(foo='one', bar=None)
        utils.validate_mutually_exclusive(parsed, ['foo'], ['bar'])
        # If bar is specified and foo is not, then we're fine.
        parsed = FakeParsedArgs(foo=None, bar='one')
        utils.validate_mutually_exclusive(parsed, ['foo'], ['bar'])
        # But if we specify both foo and bar then we get an error.
        parsed = FakeParsedArgs(foo='one', bar='two')
        with self.assertRaises(ParamValidationError):
            utils.validate_mutually_exclusive(parsed, ['foo'], ['bar'])

    def test_multiple_groups(self):
        groups = (
            ['one', 'two', 'three'],
            ['foo', 'bar', 'baz'],
            ['qux', 'bad', 'morebad'],
        )
        # This is fine.
        parsed = FakeParsedArgs(foo='foo', bar='bar', baz='baz')
        utils.validate_mutually_exclusive(parsed, *groups)
        # But this is bad.
        parsed = FakeParsedArgs(foo='one', bar=None, qux='three')
        with self.assertRaises(ParamValidationError):
            utils.validate_mutually_exclusive(parsed, *groups)


class TestS3BucketExists(unittest.TestCase):
    def setUp(self):
        self.s3_client = mock.Mock()
        self.bucket_name = 'mybucket'
        self.error_response = {
            'Error': {'Code': '404', 'Message': 'Not Found'}
        }
        self.bucket_no_exists_error = ClientError(
            self.error_response, 'HeadBucket'
        )

    def test_bucket_exists(self):
        self.assertTrue(
            utils.s3_bucket_exists(self.s3_client, self.bucket_name)
        )

    def test_bucket_not_exists(self):
        self.s3_client.head_bucket.side_effect = self.bucket_no_exists_error
        self.assertFalse(
            utils.s3_bucket_exists(self.s3_client, self.bucket_name)
        )

    def test_bucket_exists_with_non_404(self):
        self.error_response['Error']['Code'] = '403'
        self.error_response['Error']['Message'] = 'Forbidden'
        forbidden_error = ClientError(self.error_response, 'HeadBucket')
        self.s3_client.head_bucket.side_effect = forbidden_error
        self.assertTrue(
            utils.s3_bucket_exists(self.s3_client, self.bucket_name)
        )


class TestClientCreationFromGlobals(unittest.TestCase):
    def setUp(self):
        self.fake_client = {}
        self.session = mock.Mock()
        self.session.create_client.return_value = self.fake_client
        self.parsed_globals = argparse.Namespace()
        self.parsed_globals.region = 'us-west-2'
        self.parsed_globals.endpoint_url = 'https://foo.bar.com'
        self.parsed_globals.verify_ssl = False

    def test_creates_clients_with_no_overrides(self):
        client = utils.create_client_from_parsed_globals(
            self.session, 'ec2', self.parsed_globals
        )
        self.assertEqual(self.fake_client, client)
        self.session.create_client.assert_called_once_with(
            'ec2',
            region_name='us-west-2',
            verify=False,
            endpoint_url='https://foo.bar.com',
        )

    def test_creates_clients_with_overrides(self):
        overrides = {
            'region_name': 'custom',
            'verify': True,
            'other_thing': 'more custom',
        }
        client = utils.create_client_from_parsed_globals(
            self.session, 'ec2', self.parsed_globals, overrides
        )
        self.assertEqual(self.fake_client, client)
        self.session.create_client.assert_called_once_with(
            'ec2',
            region_name='custom',
            verify=True,
            other_thing='more custom',
            endpoint_url='https://foo.bar.com',
        )

    def test_creates_clients_with_no_parsed_globals(self):
        client = utils.create_client_from_parsed_globals(
            self.session, 'ec2', argparse.Namespace()
        )
        self.assertEqual(self.fake_client, client)
        self.session.create_client.assert_called_once_with('ec2')


class MockPipedStdout(io.BytesIO):
    """Mocks `sys.stdout`.

    We can't use `TextIOWrapper` because calling
    `TextIOWrapper(.., encoding=None)` sets the ``encoding`` attribute to
    `UTF-8`. The attribute is also `readonly` in `TextIOWrapper` and
    `TextIOBase` so it cannot be overwritten in subclasses.
    """

    def __init__(self):
        self.encoding = None

        super(MockPipedStdout, self).__init__()

    def write(self, data):
        # sys.stdout.write() will default to encoding to ascii, when its
        # `encoding` is `None`.
        if self.encoding is None:
            data = data.encode('ascii')
        else:
            data = data.encode(self.encoding)
        super(MockPipedStdout, self).write(data)


class TestUniPrint(unittest.TestCase):
    def test_out_file_with_encoding_attribute(self):
        buf = io.BytesIO()
        out = io.TextIOWrapper(buf, encoding='utf-8')
        utils.uni_print('\u2713', out)
        self.assertEqual(buf.getvalue(), '\u2713'.encode())

    def test_encoding_with_encoding_none(self):
        '''When the output of the aws command is being piped,
        the `encoding` attribute of `sys.stdout` is `None`.'''
        out = MockPipedStdout()
        utils.uni_print('SomeChars\u2713\u2714OtherChars', out)
        self.assertEqual(out.getvalue(), b'SomeChars??OtherChars')

    def test_encoding_statement_fails_are_replaced(self):
        buf = io.BytesIO()
        out = io.TextIOWrapper(buf, encoding='ascii')
        utils.uni_print('SomeChars\u2713\u2714OtherChars', out)
        # We replace the characters that can't be encoded
        # with '?'.
        self.assertEqual(buf.getvalue(), b'SomeChars??OtherChars')


class TestGetPolicyARNSuffix(unittest.TestCase):
    def test_get_policy_arn_suffix(self):
        self.assertEqual(
            "aws-cn", utils.get_policy_arn_suffix("cn-northwest-1")
        )
        self.assertEqual(
            "aws-cn", utils.get_policy_arn_suffix("cn-northwest-2")
        )
        self.assertEqual("aws-cn", utils.get_policy_arn_suffix("cn-north-1"))
        self.assertEqual(
            "aws-us-gov", utils.get_policy_arn_suffix("us-gov-west-1")
        )
        self.assertEqual("aws", utils.get_policy_arn_suffix("ca-central-1"))
        self.assertEqual("aws", utils.get_policy_arn_suffix("us-east-1"))
        self.assertEqual("aws", utils.get_policy_arn_suffix("sa-east-1"))
        self.assertEqual("aws", utils.get_policy_arn_suffix("ap-south-1"))


class TestGetShapeDocOverview(unittest.TestCase):
    def assert_expected_shape_overview(self, shape_docs, expected_overview):
        shape = mock.Mock(Shape)
        shape.documentation = shape_docs
        actual_overview = utils.get_shape_doc_overview(shape)
        self.assertEqual(actual_overview, expected_overview)

    def test_get_shape_doc_overview(self):
        self.assert_expected_shape_overview(
            shape_docs='Shape documentation',
            expected_overview='Shape documentation.',
        )

    def test_uses_content_before_first_period(self):
        self.assert_expected_shape_overview(
            shape_docs='First sentence. Second Sentence.',
            expected_overview='First sentence.',
        )

    def test_uses_content_before_first_colon(self):
        self.assert_expected_shape_overview(
            shape_docs='<p>Broken XML docs',
            expected_overview='<p>Broken XML docs.',
        )

    def test_removes_xml_tags(self):
        self.assert_expected_shape_overview(
            shape_docs='<p>Shape documentation</p>',
            expected_overview='Shape documentation.',
        )

    def test_removes_nested_xml_tags(self):
        self.assert_expected_shape_overview(
            shape_docs='<p>Shape <code>documentation</code></p>',
            expected_overview='Shape documentation.',
        )

    def test_can_handle_broken_xml(self):
        self.assert_expected_shape_overview(
            shape_docs='<p>Broken XML docs',
            expected_overview='<p>Broken XML docs.',
        )

    def test_ignores_newlines(self):
        self.assert_expected_shape_overview(
            shape_docs='First line\nSecond line',
            expected_overview='First line Second line.',
        )

    def test_ignores_line_separator_char(self):
        self.assert_expected_shape_overview(
            shape_docs='First line\u2028Second line',
            expected_overview='First line Second line.',
        )



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from datetime import datetime
from time import mktime

from tests.unit.customizations.emr import (
    EMRBaseAWSCommandParamsTest as BaseAWSCommandParamsTest,
)


class TestListClusters(BaseAWSCommandParamsTest):
    prefix = 'emr list-clusters '

    def test_list_active_clusters(self):
        args = '--active'
        cmdline = self.prefix + args
        result = {
            'ClusterStates': [
                'STARTING',
                'BOOTSTRAPPING',
                'RUNNING',
                'WAITING',
                'TERMINATING',
            ]
        }
        self.assert_params_for_cmd(cmdline, result)

    def test_list_terminated_clusters(self):
        args = '--terminated'
        cmdline = self.prefix + args
        result = {'ClusterStates': ['TERMINATED']}
        self.assert_params_for_cmd(cmdline, result)

    def test_list_failed_clusters(self):
        args = '--failed'
        cmdline = self.prefix + args
        result = {'ClusterStates': ['TERMINATED_WITH_ERRORS']}
        self.assert_params_for_cmd(cmdline, result)

    def test_list_multiple_states(self):
        args = '--cluster-states RUNNING WAITING TERMINATED'
        cmdline = self.prefix + args
        result = {'ClusterStates': ['RUNNING', 'WAITING', 'TERMINATED']}
        self.assert_params_for_cmd(cmdline, result)

    def test_exclusive_states_filters(self):
        args = '--active --failed'
        cmdline = self.prefix + args
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): You can specify only one of the cluster state '
            'filters: --cluster-states, --active, --terminated, --failed.\n'
        )
        result = self.run_cmd(cmdline, 252)
        self.assertEqual(expected_error_msg, result[1])

        args = '--cluster-states STARTING RUNNING --terminated'
        cmdline = self.prefix + args
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): You can specify only one of the cluster state '
            'filters: --cluster-states, --active, --terminated, --failed.\n'
        )
        result = self.run_cmd(cmdline, 252)
        self.assertEqual(expected_error_msg, result[1])


if __name__ == "__main__":
    unittest.main()



# Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest, mock


class TestStartLiveTailCommand(BaseAWSCommandParamsTest):
    def setUp(self):
        super(TestStartLiveTailCommand, self).setUp()
        self.log_group_identifiers = [
            "arn:aws:logs:us-east-1:123456789012:log-group:mygroup"
        ]
        self.log_stream_names = ["mystream"]
        self.log_stream_name_prefixes = ["mystream"]
        self.log_event_filter_pattern = "filterPattern"
        self.interactive = True

        self.raw_stream = mock.Mock()
        self.raw_stream.stream = mock.MagicMock(return_value=[])
        self.event_stream = mock.Mock()
        self.updates = iter(
            [
                {"sessionStart": "The session is started"},
                {
                    "sessionUpdate": {
                        "sessionMetadata": {"sampled": False},
                        "sessionResults": [
                            {"message": "LogEvent1"},
                            {"message": "LogEvent2"},
                        ],
                    }
                },
            ]
        )
        self.parsed_responses = [{"responseStream": self.event_stream}]

    def tearDown(self):
        super(TestStartLiveTailCommand, self).tearDown()

    def test_start_live_tail(self):
        self.event_stream.close = mock.MagicMock()
        self.event_stream.__iter__ = mock.MagicMock(return_value=self.updates)

        stdout, _, _ = self.assert_params_for_cmd(
            "logs start-live-tail --log-group-identifiers {}".format(
                " ".join(self.log_group_identifiers)
            ),
            params={"logGroupIdentifiers": self.log_group_identifiers},
        )
        self.assertEqual(stdout, "LogEvent1\nLogEvent2\n")

    def test_start_live_tail_with_log_stream_names(self):
        self.event_stream.close = mock.MagicMock()
        self.event_stream.__iter__ = mock.MagicMock(return_value=self.updates)

        stdout, _, _ = self.assert_params_for_cmd(
            "logs start-live-tail --log-group-identifiers {} --log-stream-names {}".format(
                " ".join(self.log_group_identifiers),
                " ".join(self.log_stream_names),
            ),
            params={
                "logGroupIdentifiers": self.log_group_identifiers,
                "logStreamNames": self.log_stream_names,
            },
        )
        self.assertEqual(stdout, "LogEvent1\nLogEvent2\n")

    def test_start_live_tail_with_log_stream_name_prefixes(self):
        self.event_stream.close = mock.MagicMock()
        self.event_stream.__iter__ = mock.MagicMock(return_value=self.updates)

        stdout, _, _ = self.assert_params_for_cmd(
            "logs start-live-tail --log-group-identifiers {} --log-stream-name-prefixes {}".format(
                " ".join(self.log_group_identifiers),
                " ".join(self.log_stream_name_prefixes),
            ),
            params={
                "logGroupIdentifiers": self.log_group_identifiers,
                "logStreamNamePrefixes": self.log_stream_name_prefixes,
            },
        )
        self.assertEqual(stdout, "LogEvent1\nLogEvent2\n")

    def test_start_live_tail_with_filter_pattern(self):
        self.event_stream.close = mock.MagicMock()
        self.event_stream.__iter__ = mock.MagicMock(return_value=self.updates)

        stdout, _, _ = self.assert_params_for_cmd(
            "logs start-live-tail --log-group-identifiers {} --log-event-filter-pattern {}".format(
                " ".join(self.log_group_identifiers),
                self.log_event_filter_pattern,
            ),
            params={
                "logGroupIdentifiers": self.log_group_identifiers,
                "logEventFilterPattern": self.log_event_filter_pattern,
            },
        )
        self.assertEqual(stdout, "LogEvent1\nLogEvent2\n")



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import copy
import os

from awscli.testutils import mock
from tests.unit.customizations.emr import (
    EMRBaseAWSCommandParamsTest as BaseAWSCommandParamsTest,
)


class TestAddSteps(BaseAWSCommandParamsTest):
    prefix = 'emr add-steps --cluster-id j-ABC --steps '

    STREAMING_ARGS = (
        'Args=-files,'
        + 's3://elasticmapreduce/samples/wordcount/wordSplitter.py,'
        + '-mapper,wordSplitter.py,'
        + '-reducer,aggregate,'
        + '-input,s3://elasticmapreduce/samples/wordcount/input,'
        + '-output,s3://mybucket/wordcount/output/2014-04-18/12-15-24'
    )
    STREAMING_HADOOP_SCRIPT_RUNNER_STEP = {
        'Jar': '/home/hadoop/contrib/streaming/hadoop-streaming.jar',
        'Args': [
            '-files',
            's3://elasticmapreduce/samples/wordcount/wordSplitter.py',
            '-mapper',
            'wordSplitter.py',
            '-reducer',
            'aggregate',
            '-input',
            's3://elasticmapreduce/samples/wordcount/input',
            '-output',
            's3://mybucket/wordcount/output/2014-04-18/12-15-24',
        ],
    }
    STREAMING_HADOOP_COMMAND_RUNNER_STEP = {
        'Jar': 'command-runner.jar',
        'Args': [
            'hadoop-streaming',
            '-files',
            's3://elasticmapreduce/samples/wordcount/wordSplitter.py',
            '-mapper',
            'wordSplitter.py',
            '-reducer',
            'aggregate',
            '-input',
            's3://elasticmapreduce/samples/wordcount/input',
            '-output',
            's3://mybucket/wordcount/output/2014-04-18/12-15-24',
        ],
    }

    HIVE_BASIC_ARGS = (
        'Args=-f,'
        + 's3://elasticmapreduce/samples/hive-ads/libs/model-build.q'
    )

    HIVE_DEFAULT_SCRIPT_RUNNER_STEP = {
        'Jar': (
            's3://us-east-1.elasticmapreduce/'
            'libs/script-runner/script-runner.jar'
        ),
        'Args': [
            's3://us-east-1.elasticmapreduce/libs/hive/hive-script',
            '--run-hive-script',
            '--hive-versions',
            'latest',
            '--args',
            '-f',
            's3://elasticmapreduce/samples/hive-ads/libs/model-build.q',
        ],
    }

    HIVE_DEFAULT_COMMAND_RUNNER_STEP = {
        'Jar': ('command-runner.jar'),
        'Args': [
            'hive-script',
            '--run-hive-script',
            '--args',
            '-f',
            's3://elasticmapreduce/samples/hive-ads/libs/model-build.q',
        ],
    }

    PIG_BASIC_ARGS = (
        'Args=-f,' + 's3://elasticmapreduce/samples/pig-apache/do-reports2.pig'
    )

    PIG_DEFAULT_SCRIPT_RUNNER_STEP = {
        'Jar': (
            's3://us-east-1.elasticmapreduce/libs/'
            'script-runner/script-runner.jar'
        ),
        'Args': [
            's3://us-east-1.elasticmapreduce/libs/pig/pig-script',
            '--run-pig-script',
            '--pig-versions',
            'latest',
            '--args',
            '-f',
            's3://elasticmapreduce/samples/' 'pig-apache/do-reports2.pig',
        ],
    }

    PIG_DEFAULT_COMMAND_RUNNER_STEP = {
        'Jar': ('command-runner.jar'),
        'Args': [
            'pig-script',
            '--run-pig-script',
            '--args',
            '-f',
            's3://elasticmapreduce/samples/' 'pig-apache/do-reports2.pig',
        ],
    }

    IMPALA_BASIC_ARGS = (
        'Args='
        + '--impala-script,s3://myimpala/input,'
        + '--console-output-path,s3://myimpala/output'
    )

    IMPALA_BASIC_SCRIPT_RUNNER_STEP = {
        'Jar': (
            's3://us-east-1.elasticmapreduce/libs/'
            'script-runner/script-runner.jar'
        ),
        'Args': [
            's3://us-east-1.elasticmapreduce/libs/impala/setup-impala',
            '--run-impala-script',
            '--impala-script',
            's3://myimpala/input',
            '--console-output-path',
            's3://myimpala/output',
        ],
    }

    SPARK_SUBMIT_BASIC_ARGS = (
        'Args='
        + '[--deploy-mode,'
        + 'cluster,'
        + '--conf,'
        + 'k1=v1,'
        + 's3://mybucket/myfolder/app.jar,'
        + 'k2=v2]'
    )

    SPARK_SUBMIT_SCRIPT_RUNNER_STEP = {
        'Jar': (
            's3://us-east-1.elasticmapreduce/libs/'
            'script-runner/script-runner.jar'
        ),
        'Args': [
            '/home/hadoop/spark/bin/spark-submit',
            '--deploy-mode',
            'cluster',
            '--conf',
            'k1=v1',
            's3://mybucket/myfolder/app.jar',
            'k2=v2',
        ],
    }

    SPARK_SUBMIT_COMMAND_RUNNER_STEP = {
        'Jar': 'command-runner.jar',
        'Args': [
            'spark-submit',
            '--deploy-mode',
            'cluster',
            '--conf',
            'k1=v1',
            's3://mybucket/myfolder/app.jar',
            'k2=v2',
        ],
    }

    def test_unknown_step_type(self):
        cmd = self.prefix + 'Type=unknown'
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): '
            + 'The step type unknown is not supported.\n'
        )
        self.assert_error_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_error_msg=expected_error_msg,
            expected_result_release=expected_error_msg,
        )

    def test_default_step_type_name_action_on_failure(self):
        cmd = self.prefix + 'Jar=s3://mybucket/mytest.jar'
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Custom JAR',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': {'Jar': 's3://mybucket/mytest.jar'},
                }
            ],
        }

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result,
        )

    def test_custom_jar_step_missing_jar(self):
        cmd = self.prefix + 'Name=CustomJarMissingJar'
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): The following '
            + 'required parameters are missing for CustomJARStepConfig: Jar.\n'
        )
        self.assert_error_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_error_msg=expected_error_msg,
            expected_result_release=expected_error_msg,
        )

    def test_custom_jar_step_with_all_fields(self):
        cmd = self.prefix + (
            'Name=Custom,Type=Custom_jar,'
            'Jar=s3://mybucket/mytest.jar,'
            'Args=arg1,arg2,MainClass=mymainclass,'
            'ActionOnFailure=TERMINATE_CLUSTER,'
            'LogUri="TestLogUri",'
            'EncryptionKeyArn="TestEncryptionKeyArn",'
            'Properties=k1=v1\\,k2=v2\\,k3'
        )
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Custom',
                    'ActionOnFailure': 'TERMINATE_CLUSTER',
                    'HadoopJarStep': {
                        'Jar': 's3://mybucket/mytest.jar',
                        'Args': ['arg1', 'arg2'],
                        'MainClass': 'mymainclass',
                        'Properties': [
                            {'Key': 'k1', 'Value': 'v1'},
                            {'Key': 'k2', 'Value': 'v2'},
                            {'Key': 'k3', 'Value': ''},
                        ],
                    },
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {
                            'LogUri': "TestLogUri",
                            'EncryptionKeyArn': "TestEncryptionKeyArn",
                        }
                    },
                }
            ],
        }

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result,
        )

    def test_custom_jar_step_with_step_monitoring_configuration_log_uri_only(
        self,
    ):
        cmd = self.prefix + (
            'Name=Custom,Type=Custom_jar,'
            'Jar=s3://mybucket/mytest.jar,'
            'Args=arg1,arg2,MainClass=mymainclass,'
            'ActionOnFailure=TERMINATE_CLUSTER,'
            'LogUri="TestLogUri"'
        )
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Custom',
                    'ActionOnFailure': 'TERMINATE_CLUSTER',
                    'HadoopJarStep': {
                        'Jar': 's3://mybucket/mytest.jar',
                        'Args': ['arg1', 'arg2'],
                        'MainClass': 'mymainclass',
                    },
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {'LogUri': "TestLogUri"}
                    },
                }
            ],
        }

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result,
        )

    def test_custom_jar_step_with_step_monitoring_configuration_encryption_key_arn_only(
        self,
    ):
        cmd = self.prefix + (
            'Name=Custom,Type=Custom_jar,'
            'Jar=s3://mybucket/mytest.jar,'
            'Args=arg1,arg2,MainClass=mymainclass,'
            'ActionOnFailure=TERMINATE_CLUSTER,'
            'EncryptionKeyArn="TestEncryptionKeyArn"'
        )
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Custom',
                    'ActionOnFailure': 'TERMINATE_CLUSTER',
                    'HadoopJarStep': {
                        'Jar': 's3://mybucket/mytest.jar',
                        'Args': ['arg1', 'arg2'],
                        'MainClass': 'mymainclass',
                    },
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {
                            'EncryptionKeyArn': "TestEncryptionKeyArn"
                        }
                    },
                }
            ],
        }

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result,
        )

    def test_custom_jar_step_with_step_monitoring_configuration_no_log_uri_or_encryption_key_arn(
        self,
    ):
        cmd = self.prefix + (
            'Name=Custom,Type=Custom_jar,'
            'Jar=s3://mybucket/mytest.jar,'
            'Args=arg1,arg2,MainClass=mymainclass,'
            'ActionOnFailure=TERMINATE_CLUSTER'
        )
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Custom',
                    'ActionOnFailure': 'TERMINATE_CLUSTER',
                    'HadoopJarStep': {
                        'Jar': 's3://mybucket/mytest.jar',
                        'Args': ['arg1', 'arg2'],
                        'MainClass': 'mymainclass',
                    },
                }
            ],
        }

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result,
        )

    def test_streaming_step_with_default_fields(self):
        cmd = self.prefix + 'Type=Streaming,' + self.STREAMING_ARGS
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Streaming program',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.STREAMING_HADOOP_SCRIPT_RUNNER_STEP,
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.STREAMING_HADOOP_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_step_with_execution_role_arn(self):
        cmd = self.prefix + 'Type=Streaming,' + self.STREAMING_ARGS
        cmd += ' --execution-role-arn arn:aws:iam::123456789010:role/sample '
        expected_result = {
            'ExecutionRoleArn': 'arn:aws:iam::123456789010:role/sample',
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Streaming program',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.STREAMING_HADOOP_SCRIPT_RUNNER_STEP,
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.STREAMING_HADOOP_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_streaming_step_missing_args(self):
        cmd = self.prefix + 'Type=Streaming'
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): The following '
            + 'required parameters are missing for StreamingStepConfig: Args.\n'
        )
        self.assert_error_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_error_msg=expected_error_msg,
            expected_result_release=expected_error_msg,
        )

    def test_streaming_jar_with_all_fields(self):
        test_step_config = (
            'Type=Streaming,'
            + 'Name=StreamingStepAllFields,'
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + self.STREAMING_ARGS
            + ','
            + 'LogUri="TestLogUri",'
            + 'EncryptionKeyArn="TestEncryptionKeyArn"'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'StreamingStepAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.STREAMING_HADOOP_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {
                            'LogUri': "TestLogUri",
                            'EncryptionKeyArn': "TestEncryptionKeyArn",
                        }
                    },
                }
            ],
        }

        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.STREAMING_HADOOP_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_streaming_jar_with_step_monitoring_configuration_log_uri_only(
        self,
    ):
        test_step_config = (
            'Type=Streaming,'
            + 'Name=StreamingStepAllFields,'
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + self.STREAMING_ARGS
            + ','
            + 'LogUri="TestLogUri"'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'StreamingStepAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.STREAMING_HADOOP_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {'LogUri': "TestLogUri"}
                    },
                }
            ],
        }

        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.STREAMING_HADOOP_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_streaming_jar_with_step_monitoring_configuration_encryption_key_arn_only(
        self,
    ):
        test_step_config = (
            'Type=Streaming,'
            + 'Name=StreamingStepAllFields,'
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + self.STREAMING_ARGS
            + ','
            + 'EncryptionKeyArn="TestEncryptionKeyArn"'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'StreamingStepAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.STREAMING_HADOOP_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {
                            'EncryptionKeyArn': "TestEncryptionKeyArn"
                        }
                    },
                }
            ],
        }

        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.STREAMING_HADOOP_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_streaming_jar_with_step_monitoring_configuration_no_log_uri_or_encryption_key_arn(
        self,
    ):
        test_step_config = (
            'Type=Streaming,'
            + 'Name=StreamingStepAllFields,'
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + self.STREAMING_ARGS
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'StreamingStepAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.STREAMING_HADOOP_SCRIPT_RUNNER_STEP,
                }
            ],
        }

        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.STREAMING_HADOOP_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_hive_step_with_default_fields(self):
        cmd = self.prefix + 'Type=Hive,' + self.HIVE_BASIC_ARGS
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Hive program',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.HIVE_DEFAULT_SCRIPT_RUNNER_STEP,
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.HIVE_DEFAULT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_hive_step_missing_args(self):
        cmd = self.prefix + 'Type=Hive'
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): The following '
            + 'required parameters are missing for HiveStepConfig: Args.\n'
        )

        self.assert_error_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_error_msg=expected_error_msg,
            expected_result_release=expected_error_msg,
        )

    def test_hive_step_with_all_fields(self):
        test_step_config = (
            'Type=Hive,'
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + 'Name=HiveWithAllFields,'
            + self.HIVE_BASIC_ARGS
            + ','
            + 'LogUri="TestLogUri",'
            + 'EncryptionKeyArn="TestEncryptionKeyArn"'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'HiveWithAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.HIVE_DEFAULT_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {
                            'LogUri': "TestLogUri",
                            'EncryptionKeyArn': "TestEncryptionKeyArn",
                        }
                    },
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.HIVE_DEFAULT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_hive_step_with_step_monitoring_configuration_log_uri_only(self):
        test_step_config = (
            'Type=Hive,'
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + 'Name=HiveWithAllFields,'
            + self.HIVE_BASIC_ARGS
            + ','
            + 'LogUri="TestLogUri"'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'HiveWithAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.HIVE_DEFAULT_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {
                            'LogUri': "TestLogUri",
                        }
                    },
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.HIVE_DEFAULT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_hive_step_with_step_monitoring_configuration_encryption_key_arn_only(
        self,
    ):
        test_step_config = (
            'Type=Hive,'
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + 'Name=HiveWithAllFields,'
            + self.HIVE_BASIC_ARGS
            + ','
            + 'EncryptionKeyArn="TestEncryptionKeyArn"'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'HiveWithAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.HIVE_DEFAULT_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {
                            'EncryptionKeyArn': "TestEncryptionKeyArn"
                        }
                    },
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.HIVE_DEFAULT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_hive_step_with_step_monitoring_configuration_no_log_uri_or_encryption_key_arn(
        self,
    ):
        test_step_config = (
            'Type=Hive,'
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + 'Name=HiveWithAllFields,'
            + self.HIVE_BASIC_ARGS
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'HiveWithAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.HIVE_DEFAULT_SCRIPT_RUNNER_STEP,
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.HIVE_DEFAULT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_pig_step_with_default_fields(self):
        cmd = self.prefix + 'Type=Pig,' + self.PIG_BASIC_ARGS
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Pig program',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.PIG_DEFAULT_SCRIPT_RUNNER_STEP,
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.PIG_DEFAULT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_pig_missing_args(self):
        cmd = self.prefix + 'Type=Pig'
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): The following '
            + 'required parameters are missing for PigStepConfig: Args.\n'
        )
        self.assert_error_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_error_msg=expected_error_msg,
            expected_result_release=expected_error_msg,
        )

    def test_pig_step_with_all_fields(self):
        test_step_config = (
            'Name=PigWithAllFields,'
            + 'Type=Pig,'
            + self.PIG_BASIC_ARGS
            + ','
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + 'LogUri="TestLogUri",'
            + 'EncryptionKeyArn="TestEncryptionKeyArn"'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'PigWithAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.PIG_DEFAULT_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {
                            'LogUri': "TestLogUri",
                            'EncryptionKeyArn': "TestEncryptionKeyArn",
                        }
                    },
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.PIG_DEFAULT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_pig_step_with_step_monitoring_configuration_log_uri_only(self):
        test_step_config = (
            'Name=PigWithAllFields,'
            + 'Type=Pig,'
            + self.PIG_BASIC_ARGS
            + ','
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + 'LogUri="TestLogUri"'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'PigWithAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.PIG_DEFAULT_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {'LogUri': "TestLogUri"}
                    },
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.PIG_DEFAULT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_pig_step_with_step_monitoring_configuration_encryption_key_arn_only(
        self,
    ):
        test_step_config = (
            'Name=PigWithAllFields,'
            + 'Type=Pig,'
            + self.PIG_BASIC_ARGS
            + ','
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + 'EncryptionKeyArn="TestEncryptionKeyArn"'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'PigWithAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.PIG_DEFAULT_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {
                            'EncryptionKeyArn': "TestEncryptionKeyArn"
                        }
                    },
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.PIG_DEFAULT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_pig_step_with_step_monitoring_configuration_no_log_uri_or_encryption_key_arn(
        self,
    ):
        test_step_config = (
            'Name=PigWithAllFields,'
            + 'Type=Pig,'
            + self.PIG_BASIC_ARGS
            + ','
            + 'ActionOnFailure=CANCEL_AND_WAIT'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'PigWithAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.PIG_DEFAULT_SCRIPT_RUNNER_STEP,
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.PIG_DEFAULT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_impala_step_with_default_fields(self):
        test_step_config = 'Type=Impala,' + self.IMPALA_BASIC_ARGS
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Impala program',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.IMPALA_BASIC_SCRIPT_RUNNER_STEP,
                }
            ],
        }
        self.assert_params_for_cmd(cmd, expected_result)

    def test_SPARK_SUBMIT_SCRIPT_RUNNER_STEP(self):
        cmd = self.prefix + 'Type=SPARK,' + self.SPARK_SUBMIT_BASIC_ARGS
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Spark application',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.SPARK_SUBMIT_SCRIPT_RUNNER_STEP,
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.SPARK_SUBMIT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_spark_missing_arg(self):
        cmd = self.prefix + 'Type=SPARK'
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): The following '
            + 'required parameters are missing for SparkStepConfig: Args.\n'
        )
        self.assert_error_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_error_msg=expected_error_msg,
            expected_result_release=expected_error_msg,
        )

    def test_spark_step_with_step_monitoring_configuration(self):
        cmd = (
            self.prefix
            + 'Type=SPARK,'
            + self.SPARK_SUBMIT_BASIC_ARGS
            + ','
            + 'LogUri="TestLogUri",'
            + 'EncryptionKeyArn="TestEncryptionKeyArn"'
        )
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Spark application',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.SPARK_SUBMIT_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {
                            'LogUri': "TestLogUri",
                            'EncryptionKeyArn': "TestEncryptionKeyArn",
                        }
                    },
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.SPARK_SUBMIT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_spark_step_with_step_monitoring_configuration_log_uri_only(self):
        cmd = (
            self.prefix
            + 'Type=SPARK,'
            + self.SPARK_SUBMIT_BASIC_ARGS
            + ','
            + 'LogUri="TestLogUri"'
        )
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Spark application',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.SPARK_SUBMIT_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {'LogUri': "TestLogUri"}
                    },
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.SPARK_SUBMIT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_spark_step_with_step_monitoring_configuration_encryption_key_arn_only(
        self,
    ):
        cmd = (
            self.prefix
            + 'Type=SPARK,'
            + self.SPARK_SUBMIT_BASIC_ARGS
            + ','
            + 'EncryptionKeyArn="TestEncryptionKeyArn"'
        )
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Spark application',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.SPARK_SUBMIT_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {
                            'EncryptionKeyArn': "TestEncryptionKeyArn"
                        }
                    },
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.SPARK_SUBMIT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_spark_step_with_step_monitoring_configuration_no_log_uri_or_encryption_key_arn(
        self,
    ):
        cmd = self.prefix + 'Type=SPARK,' + self.SPARK_SUBMIT_BASIC_ARGS
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Spark application',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.SPARK_SUBMIT_SCRIPT_RUNNER_STEP,
                }
            ],
        }
        expected_result_release = copy.deepcopy(expected_result)
        expected_result_release['Steps'][0]['HadoopJarStep'] = (
            self.SPARK_SUBMIT_COMMAND_RUNNER_STEP
        )

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=expected_result_release,
        )

    def test_impala_missing_args(self):
        cmd = self.prefix + 'Type=Impala'
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): The following '
            + 'required parameters are missing for ImpalaStepConfig: Args.\n'
        )
        self.assert_error_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_error_msg=expected_error_msg,
            expected_result_release=None,
        )

    def test_impala_step_with_all_fields(self):
        test_step_config = (
            'Name=ImpalaWithAllFields,'
            + 'Type=Impala,'
            + self.IMPALA_BASIC_ARGS
            + ','
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + 'LogUri="TestLogUri",'
            + 'EncryptionKeyArn="TestEncryptionKeyArn"'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'ImpalaWithAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.IMPALA_BASIC_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {
                            'LogUri': "TestLogUri",
                            'EncryptionKeyArn': "TestEncryptionKeyArn",
                        }
                    },
                }
            ],
        }
        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=None,
        )

    def test_impala_step_with_step_monitoring_configuration_log_uri_only(self):
        test_step_config = (
            'Name=ImpalaWithAllFields,'
            + 'Type=Impala,'
            + self.IMPALA_BASIC_ARGS
            + ','
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + 'LogUri="TestLogUri"'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'ImpalaWithAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.IMPALA_BASIC_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {'LogUri': "TestLogUri"}
                    },
                }
            ],
        }
        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=None,
        )

    def test_impala_step_with_step_monitoring_configuration_encryption_key_arn_only(
        self,
    ):
        test_step_config = (
            'Name=ImpalaWithAllFields,'
            + 'Type=Impala,'
            + self.IMPALA_BASIC_ARGS
            + ','
            + 'ActionOnFailure=CANCEL_AND_WAIT,'
            + 'EncryptionKeyArn="TestEncryptionKeyArn"'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'ImpalaWithAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.IMPALA_BASIC_SCRIPT_RUNNER_STEP,
                    'StepMonitoringConfiguration': {
                        'S3MonitoringConfiguration': {
                            'EncryptionKeyArn': "TestEncryptionKeyArn"
                        }
                    },
                }
            ],
        }
        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=None,
        )

    def test_impala_step_with_step_monitoring_configuration_no_log_uri_or_encryption_key_arn(
        self,
    ):
        test_step_config = (
            'Name=ImpalaWithAllFields,'
            + 'Type=Impala,'
            + self.IMPALA_BASIC_ARGS
            + ','
            + 'ActionOnFailure=CANCEL_AND_WAIT'
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'ImpalaWithAllFields',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.IMPALA_BASIC_SCRIPT_RUNNER_STEP,
                }
            ],
        }
        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=None,
        )

    def test_impala_step_with_release(self):
        test_step_config = 'Type=Impala,' + self.IMPALA_BASIC_ARGS
        cmd = self.prefix + test_step_config
        expected_result_release = (
            '\naws: [ERROR]: An error occurred (ParamValidation): '
            + 'The step type impala is not supported.\n'
        )

        self.assert_error_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_error_msg=None,
            expected_result_release=expected_result_release,
        )

    def test_empty_step_args(self):
        cmd = self.prefix + 'Type=Streaming,Args='
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): '
            + 'The parameter Args cannot be an empty list.\n'
        )
        self.assert_error_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_error_msg=expected_error_msg,
            expected_result_release=expected_error_msg,
        )

        cmd = self.prefix + 'Type=Pig,Args='
        self.assert_error_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_error_msg=expected_error_msg,
            expected_result_release=expected_error_msg,
        )

        cmd = self.prefix + 'Type=Hive,Args='
        self.assert_error_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_error_msg=expected_error_msg,
            expected_result_release=expected_error_msg,
        )

        cmd = self.prefix + 'Args='
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): '
            'The following required parameters'
            ' are missing for CustomJARStepConfig: Jar.\n'
        )
        self.assert_error_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_error_msg=expected_error_msg,
            expected_result_release=expected_error_msg,
        )

    def test_all_step_types(self):
        test_step_config = (
            'Jar=s3://mybucket/mytest.jar '
            + ' Type=Streaming,'
            + self.STREAMING_ARGS
            + ' Type=Hive,'
            + self.HIVE_BASIC_ARGS
            + ' Type=Pig,'
            + self.PIG_BASIC_ARGS
            + ' Type=Impala,'
            + self.IMPALA_BASIC_ARGS
        )
        cmd = self.prefix + test_step_config
        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Custom JAR',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': {'Jar': 's3://mybucket/mytest.jar'},
                },
                {
                    'Name': 'Streaming program',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.STREAMING_HADOOP_SCRIPT_RUNNER_STEP,
                },
                {
                    'Name': 'Hive program',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.HIVE_DEFAULT_SCRIPT_RUNNER_STEP,
                },
                {
                    'Name': 'Pig program',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.PIG_DEFAULT_SCRIPT_RUNNER_STEP,
                },
                {
                    'Name': 'Impala program',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.IMPALA_BASIC_SCRIPT_RUNNER_STEP,
                },
            ],
        }

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=expected_result,
            expected_result_release=None,
        )

    def test_all_step_types_release(self):
        test_step_config = (
            'Jar=s3://mybucket/mytest.jar '
            + ' Type=Streaming,'
            + self.STREAMING_ARGS
            + ' Type=Hive,'
            + self.HIVE_BASIC_ARGS
            + ' Type=Pig,'
            + self.PIG_BASIC_ARGS
        )

        cmd = self.prefix + test_step_config
        expected_result_release = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Custom JAR',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': {'Jar': 's3://mybucket/mytest.jar'},
                },
                {
                    'Name': 'Streaming program',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.STREAMING_HADOOP_COMMAND_RUNNER_STEP,
                },
                {
                    'Name': 'Hive program',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.HIVE_DEFAULT_COMMAND_RUNNER_STEP,
                },
                {
                    'Name': 'Pig program',
                    'ActionOnFailure': 'CONTINUE',
                    'HadoopJarStep': self.PIG_DEFAULT_COMMAND_RUNNER_STEP,
                },
            ],
        }

        self.assert_params_for_ami_and_release_based_clusters(
            cmd=cmd,
            expected_result=None,
            expected_result_release=expected_result_release,
        )

    def test_all_step_types_from_json(self):
        data_path = os.path.join(os.path.dirname(__file__), 'input_steps.json')
        cmd = self.prefix + 'file://' + data_path
        hive_script_runner_step = copy.deepcopy(
            self.HIVE_DEFAULT_SCRIPT_RUNNER_STEP
        )
        hive_script_runner_step['Args'] += [
            '-d',
            'INPUT=s3://elasticmapreduce/samples/hive-ads/tables',
            '-d',
            'OUTPUT=s3://mybucket/hive-ads/output/2014-04-18/11-07-32',
            '-d',
            'LIBS=s3://elasticmapreduce/samples/hive-ads/libs',
        ]
        pig_script_runner_step = copy.deepcopy(
            self.PIG_DEFAULT_SCRIPT_RUNNER_STEP
        )
        pig_script_runner_step['Args'] += [
            '-p',
            'INPUT=s3://elasticmapreduce/samples/pig-apache/input',
            '-p',
            'OUTPUT=s3://mybucket/pig-apache/output/2014-04-21/20-09-28',
        ]

        expected_result = {
            'JobFlowId': 'j-ABC',
            'Steps': [
                {
                    'Name': 'Custom JAR step',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': {'Jar': 's3://mybucket/mytest.jar'},
                },
                {
                    'Name': 'Streaming step',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.STREAMING_HADOOP_SCRIPT_RUNNER_STEP,
                },
                {
                    'Name': 'Hive step',
                    'ActionOnFailure': 'TERMINATE_CLUSTER',
                    'HadoopJarStep': hive_script_runner_step,
                },
                {
                    'Name': 'Pig step',
                    'ActionOnFailure': 'TERMINATE_CLUSTER',
                    'HadoopJarStep': pig_script_runner_step,
                },
                {
                    'Name': 'Impala step',
                    'ActionOnFailure': 'CANCEL_AND_WAIT',
                    'HadoopJarStep': self.IMPALA_BASIC_SCRIPT_RUNNER_STEP,
                },
            ],
        }
        self.assert_params_for_cmd(cmd, expected_result)

    @mock.patch('awscli.customizations.emr.' 'emrutils.get_release_label')
    def assert_params_for_ami_and_release_based_clusters(
        self, grl_patch, cmd, expected_result, expected_result_release
    ):
        if expected_result:
            grl_patch.return_value = None
            self.assert_params_for_cmd(cmd, expected_result)
        if expected_result_release:
            grl_patch.return_value = 'emr-4.0'
            self.assert_params_for_cmd(cmd, expected_result_release)

    @mock.patch('awscli.customizations.emr.' 'emrutils.get_release_label')
    def assert_error_for_ami_and_release_based_clusters(
        self, grl_patch, cmd, expected_error_msg, expected_result_release
    ):
        if expected_error_msg:
            grl_patch.return_value = None
            result = self.run_cmd(cmd, 252)
            self.assertEqual(expected_error_msg, result[1])
        if expected_result_release:
            grl_patch.return_value = 'emr-4.0'
            result = self.run_cmd(cmd, 252)
            self.assertEqual(expected_result_release, result[1])


if __name__ == "__main__":
    unittest.main()



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from botocore.credentials import (
    AssumeRoleProvider,
    AssumeRoleWithWebIdentityProvider,
    CredentialResolver,
)
from botocore.exceptions import ProfileNotFound
from botocore.hooks import HierarchicalEmitter
from botocore.session import Session

from awscli.customizations import assumerole
from awscli.testutils import mock, unittest


class TestAssumeRolePlugin(unittest.TestCase):
    def test_assume_role_provider_injected(self):
        mock_assume_role = mock.Mock(spec=AssumeRoleProvider)
        mock_web_identity = mock.Mock(spec=AssumeRoleWithWebIdentityProvider)
        providers = {
            'assume-role': mock_assume_role,
            'assume-role-with-web-identity': mock_web_identity,
        }
        mock_resolver = mock.Mock(spec=CredentialResolver)
        mock_resolver.get_provider = providers.get
        session = mock.Mock(spec=Session)
        session.get_component.return_value = mock_resolver

        assumerole.inject_assume_role_provider_cache(
            session, event_name='building-command-table.foo'
        )
        session.get_component.assert_called_with('credential_provider')
        self.assertIsInstance(mock_assume_role.cache, assumerole.JSONFileCache)
        self.assertIsInstance(
            mock_web_identity.cache,
            assumerole.JSONFileCache,
        )

    def test_assume_role_provider_registration(self):
        event_handlers = HierarchicalEmitter()
        assumerole.register_assume_role_provider(event_handlers)
        session = mock.Mock(spec=Session)
        event_handlers.emit('session-initialized', session=session)
        # Just verifying that anything on the session was called ensures
        # that our handler was called, as it's the only thing that should
        # be registered.
        session.get_component.assert_called_with('credential_provider')

    def test_no_registration_if_profile_does_not_exist(self):
        session = mock.Mock(spec=Session)
        session.get_component.side_effect = ProfileNotFound(profile='unknown')

        assumerole.inject_assume_role_provider_cache(
            session, event_name='building-command-table.foo'
        )

        credential_provider = session.get_component.return_value
        self.assertFalse(credential_provider.get_provider.called)



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import glob
import os
import tempfile

from awscli.autocomplete import generator
from awscli.autocomplete.local import model
from awscli.testutils import unittest


class TestCanGenerateEntireIndex(unittest.TestCase):
    def setUp(self):
        self.temp_dir = tempfile.mkdtemp()
        self.tempfile = os.path.join(self.temp_dir, 'temp.db')
        self.model_index = model.ModelIndex(self.tempfile)

    def tearDown(self):
        self.model_index._db_connection.close()

    def assert_no_temp_index_files(self):
        # When generating the index, it generates it in a file different from
        # its final location so that only a completely built index is ever at
        # the final location. This intermediary file shares the same name as
        # final location except it has an additional suffix extension (e.g.,
        # ``.temp`` at the end). So this check ensures there are no lingering
        # intermediary files from building the index (even if the suffix
        # extension changes in the future).
        possible_matches = glob.glob(f'{self.tempfile}.*')
        self.assertEqual(possible_matches, [])

    def test_can_generate_entire_index(self):
        # The point of this test is to make sure generating the index
        # doesn't break in some obvious way.  The specifics of index
        # generation are tested in tests/unit/autocomplete.  Here we just
        # generate the entire index and perform basic sanity checks.  It's
        # just a functional smoke test.
        generator.generate_index(self.tempfile)

        # Basic sanity checks.  Index generation for the entire CLI
        # takes a while so all the sanity checks are combined in a single
        # test method.
        self.assertTrue(os.path.exists(self.tempfile))
        self.assert_no_temp_index_files()

        commands = self.model_index.command_names(lineage=['aws'])
        self.assertIn('ec2', commands)
        self.assertIn('s3', commands)
        self.assertIn('s3api', commands)
        global_args = self.model_index.arg_names(
            lineage=[], command_name='aws'
        )
        self.assertIn('region', global_args)
        self.assertIn('endpoint-url', global_args)

        single_arg = self.model_index.get_argument_data(
            lineage=[], command_name='aws', arg_name='output'
        )
        self.assertEqual(single_arg.argname, 'output')
        self.assertEqual(single_arg.command, 'aws')
        self.assertEqual(single_arg.parent, '')
        self.assertIsNone(single_arg.nargs)



import os
from pathlib import Path

from utils import bin_path, cd, run, virtualenv_enabled

INSTALL_ARGS = (
    "--no-build-isolation --no-cache-dir --no-index --prefer-binary "
)
PINNED_PIP_VERSION = '26.0.1'
SETUP_DEPS = ("setuptools-", "setuptools_scm", "wheel", "hatchling")


class InstallationError(Exception):
    pass


def get_package_tarball(package_dir, package_prefix):
    package_filenames = sorted(
        [p for p in os.listdir(package_dir) if p.startswith(package_prefix)]
    )
    if len(package_filenames) == 0:
        raise InstallationError(
            "Unable to find local package starting with %s prefix."
            % package_prefix
        )
    # We only expect a single package from the downloader
    return package_filenames[0]


def install_local_package(package_dir, package, pip_script="pip"):
    url = Path(package_dir).as_uri()
    with cd(package_dir):
        run(
            f"{pip_script} install {INSTALL_ARGS} "
            f"--find-links {url} {package}"
        )


def find_and_install_tarball(package_dir, package_prefix, pip_script="pip"):
    tarball = get_package_tarball(package_dir, package_prefix)
    install_local_package(package_dir, tarball, pip_script)


def pip_install_packages(package_dir):
    package_dir = os.path.abspath(package_dir)

    # Setup pip to support modern setuptools calls
    pip_script = os.path.join(os.environ["VIRTUAL_ENV"], bin_path(), "pip")
    local_python = os.path.join(
        os.environ["VIRTUAL_ENV"], bin_path(), "python"
    )

    # Windows can't replace a running pip.exe, so we need to work around
    run("%s -m pip install pip==%s" % (local_python, PINNED_PIP_VERSION))

    # Install or update prerequisite build packages
    setup_requires_dir = os.path.join(package_dir, "setup")
    install_setup_deps(pip_script, setup_requires_dir)

    find_and_install_tarball(package_dir, "awscli", pip_script)


def install_setup_deps(pip_script, setup_package_dir):
    # These packages need to be installed in this order before we
    # attempt anything else in order to support PEP517 setup_requires
    # specifications.

    # We need setuptools >= 37.0.0 for setuptools_scm to work properly
    for setup_dep in SETUP_DEPS:
        find_and_install_tarball(setup_package_dir, setup_dep, pip_script)


def install_packages(package_dir):
    """Builds setup environment and installs copies of local packages for CLI"""

    if not virtualenv_enabled():
        raise InstallationError(
            "Installation being performed outside of a virtualenv. Please enable before running."
        )
    else:
        pip_install_packages(package_dir)



#!/usr/bin/env python
# Copyright 2013 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestMBCommand(BaseAWSCommandParamsTest):
    prefix = 's3 mb '

    def test_make_bucket(self):
        command = self.prefix + 's3://bucket'
        self.run_cmd(command)
        self.assertEqual(len(self.operations_called), 1)
        self.assertEqual(self.operations_called[0][0].name, 'CreateBucket')

    def test_adds_location_constraint(self):
        command = self.prefix + 's3://bucket --region us-west-2'
        self.parsed_responses = [{'Location': 'us-west-2'}]
        expected_params = {
            'Bucket': 'bucket',
            'CreateBucketConfiguration': {'LocationConstraint': 'us-west-2'},
        }
        self.assert_params_for_cmd(command, expected_params)

    def test_location_constraint_not_added_on_us_east_1(self):
        command = self.prefix + 's3://bucket --region us-east-1'
        expected_params = {'Bucket': 'bucket'}
        self.assert_params_for_cmd(command, expected_params)

    def test_nonzero_exit_if_invalid_path_provided(self):
        command = self.prefix + 'bucket'
        self.run_cmd(command, expected_rc=252)

    def test_incompatible_with_express_directory_bucket(self):
        command = self.prefix + 's3://bucket--usw2-az1--x-s3/'
        stderr = self.run_cmd(command, expected_rc=252)[1]
        self.assertIn('Cannot use mb command with a directory bucket.', stderr)

    def test_make_bucket_with_single_tag(self):
        command = (
            self.prefix + 's3://bucket --tags Key1 Value1 --region us-west-2'
        )
        expected_params = {
            'Bucket': 'bucket',
            'CreateBucketConfiguration': {
                'LocationConstraint': 'us-west-2',
                'Tags': [{'Key': 'Key1', 'Value': 'Value1'}],
            },
        }
        self.assert_params_for_cmd(command, expected_params)

    def test_make_bucket_with_single_tag_us_east_1(self):
        command = (
            self.prefix + 's3://bucket --tags Key1 Value1 --region us-east-1'
        )
        expected_params = {
            'Bucket': 'bucket',
            'CreateBucketConfiguration': {
                'Tags': [{'Key': 'Key1', 'Value': 'Value1'}]
            },
        }
        self.assert_params_for_cmd(command, expected_params)

    def test_make_bucket_with_multiple_tags(self):
        command = (
            self.prefix
            + 's3://bucket --tags Key1 Value1 --tags Key2 Value2 --region us-west-2'
        )
        expected_params = {
            'Bucket': 'bucket',
            'CreateBucketConfiguration': {
                'LocationConstraint': 'us-west-2',
                'Tags': [
                    {'Key': 'Key1', 'Value': 'Value1'},
                    {'Key': 'Key2', 'Value': 'Value2'},
                ],
            },
        }
        self.assert_params_for_cmd(command, expected_params)

    def test_account_regional_namespace_bucket(self):
        bucket = 'amzn-s3-demo-bucket-111122223333-us-west-2-an'
        command = self.prefix + f's3://{bucket} --region us-west-2'
        self.parsed_responses = [{'Location': 'us-west-2'}]
        expected_params = {
            'Bucket': bucket,
            'BucketNamespace': 'account-regional',
            'CreateBucketConfiguration': {'LocationConstraint': 'us-west-2'},
        }
        self.assert_params_for_cmd(command, expected_params)

    def test_account_regional_namespace_bucket_us_east_1(self):
        bucket = 'my-bucket-111122223333-us-east-1-an'
        command = self.prefix + f's3://{bucket} --region us-east-1'
        expected_params = {
            'Bucket': bucket,
            'BucketNamespace': 'account-regional',
        }
        self.assert_params_for_cmd(command, expected_params)

    def test_account_regional_namespace_short_bucket_name(self):
        bucket = 'xyz-an'
        command = self.prefix + f's3://{bucket} --region us-east-1'
        expected_params = {
            'Bucket': bucket,
            'BucketNamespace': 'account-regional',
        }
        self.assert_params_for_cmd(command, expected_params)

    def test_regular_bucket_no_namespace(self):
        command = self.prefix + 's3://my-regular-bucket --region us-east-1'
        expected_params = {'Bucket': 'my-regular-bucket'}
        self.assert_params_for_cmd(command, expected_params)

    def test_tags_with_three_arguments_fails(self):
        command = self.prefix + 's3://bucket --tags Key1 Value1 ExtraArg'
        self.assert_params_for_cmd(
            command, expected_rc=252, stderr_contains='ParamValidation'
        )



# Copyright 2019 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from collections import namedtuple

import pytest

from awscli.autoprompt import core
from awscli.clidriver import create_clidriver
from awscli.customizations.exceptions import ParamValidationError
from awscli.testutils import mock, unittest


class TestCLIAutoPrompt(unittest.TestCase):
    def setUp(self):
        self.driver = mock.Mock()
        self.prompter = mock.Mock(spec=core.AutoPrompter)
        self.prompt_driver = core.AutoPromptDriver(
            self.driver, prompter=self.prompter
        )


class TestAutoPrompter(unittest.TestCase):
    def setUp(self):
        completion_source = mock.Mock()
        driver = mock.Mock()
        prompter = mock.Mock()
        prompter.prompt_for_args = lambda x: x
        driver.arg_table = []
        self.auto_prompter = core.AutoPrompter(
            completion_source, driver, prompter
        )

    def test_auto_prompter_returns_args(self):
        original_args = ['aws', 'ec2', 'help']
        prompted_values = self.auto_prompter.prompt_for_values(original_args)
        self.assertEqual(prompted_values, original_args)



# Copyright 2018 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.autocomplete import parser
from awscli.autocomplete.completer import CompletionResult
from awscli.autocomplete.serverside.custom_completers.logs.autocomplete import (
    GroupNameCompleter,
)
from awscli.testutils import mock, unittest
from tests.unit.autocomplete import InMemoryIndex


class TestGroupNameCompleter(unittest.TestCase):
    def setUp(self):
        self.index = InMemoryIndex(
            {
                'command_names': {
                    '': [('aws', None)],
                    'aws': [('logs', None)],
                    'aws.logs': [('tail', None)],
                },
                'arg_names': {
                    '': {
                        'aws': ['region', 'profile'],
                    },
                    'aws.logs': {
                        'tail': ['group_name'],
                    },
                },
                'arg_data': {
                    '': {
                        'aws': {
                            'profile': (
                                'profile',
                                'string',
                                'aws',
                                '',
                                None,
                                False,
                                False,
                            ),
                            'region': (
                                'region',
                                'string',
                                'aws',
                                '',
                                None,
                                False,
                                False,
                            ),
                        }
                    },
                    'aws.logs': {
                        'tail': {
                            'group_name': (
                                'group_name',
                                'string',
                                'tail',
                                'aws.logs.',
                                None,
                                True,
                                False,
                            ),
                        },
                    },
                },
            }
        )
        self.parser = parser.CLIParser(self.index)
        self.mock_client = mock.Mock()
        self.mock_create_client = mock.Mock()
        self.mock_create_client.create_client.return_value = self.mock_client
        self.completer = GroupNameCompleter(self.mock_create_client)

    def test_complete_group_name(self):
        self.mock_client.describe_log_groups.return_value = {
            'logGroups': [
                {'logGroupName': 'group'},
                {'logGroupName': 'mygroup'},
            ]
        }
        parsed = self.parser.parse('aws logs tail ')
        results = self.completer.complete(parsed)
        self.assertEqual(
            results,
            [CompletionResult('group', 0), CompletionResult('mygroup', 0)],
        )

    def test_complete_group_name_filters_startswith(self):
        self.mock_client.describe_log_groups.return_value = {
            'logGroups': [
                {'logGroupName': 'group'},
                {'logGroupName': 'mygroup'},
            ]
        }
        parsed = self.parser.parse('aws logs tail my')
        results = self.completer.complete(parsed)
        self.assertEqual(results, [CompletionResult('mygroup', -2)])

    def test_complete_group_name_handles_errors(self):
        self.mock_client.describe_log_groups.side_effect = Exception(
            "Something went wrong."
        )
        parsed = self.parser.parse('aws logs tail ')
        results = self.completer.complete(parsed)
        self.assertEqual(results, [])

    def test_client_created_with_region_and_profiles_from_parsed(self):
        self.mock_client.describe_log_groups.return_value = {'logGroups': []}
        parsed = self.parser.parse(
            'aws --profile foo --region us-west-2 logs tail '
        )
        self.completer.complete(parsed)
        self.mock_create_client.create_client.assert_called_with(
            'logs', parsed_profile='foo', parsed_region='us-west-2'
        )



# Copyright 2026 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestOTelAlias(BaseAWSCommandParamsTest):
    def test_get_otel_enrichment_alias(self):
        self.run_cmd('cloudwatch get-otel-enrichment', expected_rc=0)
        self.run_cmd('cloudwatch get-o-tel-enrichment', expected_rc=0)

    def test_start_otel_enrichment_alias(self):
        self.run_cmd('cloudwatch start-otel-enrichment', expected_rc=0)
        self.run_cmd('cloudwatch start-o-tel-enrichment', expected_rc=0)

    def test_stop_otel_enrichment_alias(self):
        self.run_cmd('cloudwatch stop-otel-enrichment', expected_rc=0)
        self.run_cmd('cloudwatch stop-o-tel-enrichment', expected_rc=0)



# Copyright 2021 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from awscli.testutils import BaseAWSCommandParamsTest


class TestCreateReplicationGroup(BaseAWSCommandParamsTest):
    GROUP_ID = 'a' * 50
    GROUP_DESCRIPTION = 'b' * 50
    PREFERRED_AZ = 'az'
    PREFIX = (
        f'elasticache create-replication-group'
        f' --replication-group-id {GROUP_ID}'
        f' --replication-group-description {GROUP_DESCRIPTION}'
    )

    def test_accepts_old_argname(self):
        cmdline = (
            f'{self.PREFIX} --preferred-cache-cluster-a-zs {self.PREFERRED_AZ}'
        )
        params = {
            'ReplicationGroupId': self.GROUP_ID,
            'ReplicationGroupDescription': self.GROUP_DESCRIPTION,
            'PreferredCacheClusterAZs': [self.PREFERRED_AZ],
        }
        self.assert_params_for_cmd(cmdline, params)

    def test_accepts_fixed_param_name(self):
        cmdline = (
            f'{self.PREFIX} --preferred-cache-cluster-azs {self.PREFERRED_AZ}'
        )
        params = {
            'ReplicationGroupId': self.GROUP_ID,
            'ReplicationGroupDescription': self.GROUP_DESCRIPTION,
            'PreferredCacheClusterAZs': [self.PREFERRED_AZ],
        }
        self.assert_params_for_cmd(cmdline, params)



# Copyright 2024 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from tests.unit.customizations.emr import (
    EMRBaseAWSCommandParamsTest as BaseAWSCommandParamsTest,
)
from tests.unit.customizations.emr import (
    test_constants_instance_fleets as CONSTANTS_FLEET,
)


class TestModifyInstanceFleet(BaseAWSCommandParamsTest):
    prefix = f"emr modify-instance-fleet --cluster-id {CONSTANTS_FLEET.DEFAULT_CLUSTER_NAME} --instance-fleet "

    def test_modify_instance_fleet_with_instance_type_configs(self):
        self.assert_params_for_cmd(
            self.prefix
            + CONSTANTS_FLEET.MODIFY_INSTANCE_FLEET_WITH_INSTANCE_TYPE_CONFIGS,
            CONSTANTS_FLEET.RES_MODIFY_INSTANCE_FLEET_WITH_INSTANCE_TYPE_CONFIGS,
        )

    def test_modify_instance_fleet_with_allocation_strategy_spot_and_od(self):
        self.assert_params_for_cmd(
            self.prefix
            + CONSTANTS_FLEET.MODIFY_INSTANCE_FLEET_WITH_SPOT_AND_OD_RESIZE_SPECIFICATIONS,
            CONSTANTS_FLEET.RES_MODIFY_INSTANCE_FLEET_WITH_SPOT_AND_OD_RESIZE_SPECIFICATIONS,
        )

    def test_modify_instance_fleet_with_allocation_strategy_spot_and_od_and_instance_type_configs(
        self,
    ):
        self.assert_params_for_cmd(
            self.prefix
            + CONSTANTS_FLEET.MODIFY_INSTANCE_FLEET_WITH_INSTANCE_TYPE_CONFIGS_AND_SPOT_AND_OD_RESIZE_SPECIFICATIONS,
            CONSTANTS_FLEET.RES_MODIFY_INSTANCE_FLEET_WITH_INSTANCE_TYPE_CONFIGS_AND_SPOT_AND_OD_RESIZE_SPECIFICATIONS,
        )


if __name__ == "__main__":
    unittest.main()



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os
from argparse import Namespace

from botocore.session import get_session

from awscli.compat import BytesIO
from awscli.customizations.gamelift.getlog import GetGameSessionLogCommand
from awscli.testutils import FileCreator, mock, unittest


class TestGetGameSessionLogCommand(unittest.TestCase):
    def setUp(self):
        self.create_client_patch = mock.patch(
            'botocore.session.Session.create_client'
        )
        self.mock_create_client = self.create_client_patch.start()
        self.session = get_session()

        self.client = mock.Mock()
        self.mock_create_client.return_value = self.client

        self.cmd = GetGameSessionLogCommand(self.session)

        self.contents = b'mycontents'
        self.file_creator = FileCreator()
        self.urlopen_patch = mock.patch(
            'awscli.customizations.gamelift.getlog.urlopen'
        )
        self.urlopen_mock = self.urlopen_patch.start()
        self.urlopen_mock.return_value = BytesIO(self.contents)

    def tearDown(self):
        self.create_client_patch.stop()
        self.file_creator.remove_all()
        self.urlopen_patch.stop()

    def test_get_game_session_log(self):
        session_id = 'mysessionid'
        save_as = os.path.join(self.file_creator.rootdir, 'mylog')

        args = ['--game-session-id', session_id, '--save-as', save_as]
        global_args = Namespace()
        global_args.region = 'us-west-2'
        global_args.endpoint_url = None
        global_args.verify_ssl = None

        presigned_url = 'mypresignedurl'
        self.client.get_game_session_log_url.return_value = {
            'PreSignedUrl': presigned_url
        }

        # Call the command
        self.cmd(args, global_args)

        # Ensure the client was created properly
        self.mock_create_client.assert_called_once_with(
            'gamelift', region_name='us-west-2', endpoint_url=None, verify=None
        )

        # Ensure the client was called correctly
        self.client.get_game_session_log_url.assert_called_once_with(
            GameSessionId=session_id
        )

        # Ensure the presigned url was used
        self.urlopen_mock.assert_called_once_with(presigned_url)

        # Ensure the contents were saved to the file
        with open(save_as, 'rb') as f:
            self.assertEqual(f.read(), self.contents)



# Copyright 2014 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
from copy import deepcopy

from awscli.testutils import mock
from tests.unit.customizations.emr import (
    EMRBaseAWSCommandParamsTest as BaseAWSCommandParamsTest,
)


class TestScheduleHBaseBackup(BaseAWSCommandParamsTest):
    prefix = 'emr schedule-hbase-backup'
    default_steps = [
        {
            'HadoopJarStep': {
                'Args': [
                    'emr.hbase.backup.Main',
                    '--set-scheduled-backup',
                    'true',
                    '--backup-dir',
                    's3://abc/',
                    '--full-backup-time-interval',
                    '10',
                    '--full-backup-time-unit',
                    'minutes',
                    '--start-time',
                    'now',
                ],
                'Jar': '/home/hadoop/lib/hbase.jar',
            },
            'Name': 'Modify Backup Schedule',
            'ActionOnFailure': 'CANCEL_AND_WAIT',
        }
    ]

    def test_schedule_hbase_backup_full(self):
        args = (
            ' --cluster-id j-ABCD --dir s3://abc/ --type full'
            + ' --interval 10 --unit minutes'
        )
        cmdline = self.prefix + args
        result = {'JobFlowId': 'j-ABCD', 'Steps': self.default_steps}

        self.assert_params_for_cmd(cmdline, result)

    def test_schedule_hbase_backup_full_upper_case(self):
        args = (
            ' --cluster-id j-ABCD --dir s3://abc/ --type FULL'
            + ' --interval 10 --unit minutes'
        )
        cmdline = self.prefix + args
        result = {'JobFlowId': 'j-ABCD', 'Steps': self.default_steps}

        self.assert_params_for_cmd(cmdline, result)

    def test_schedule_hbase_backup_incremental_upper_case(self):
        args = (
            ' --cluster-id j-ABCD --dir s3://abc/  --type INCREMENTAL'
            + ' --interval 10 --unit HOURS'
        )
        cmdline = self.prefix + args

        steps = deepcopy(self.default_steps)
        args = steps[0]['HadoopJarStep']['Args']
        args[5] = '--incremental-backup-time-interval'
        args[7] = '--incremental-backup-time-unit'
        args[8] = 'hours'
        steps[0]['HadoopJarStep']['Args'] = args
        result = {'JobFlowId': 'j-ABCD', 'Steps': steps}

        self.assert_params_for_cmd(cmdline, result)

    def test_schedule_hbase_backup_incremental(self):
        args = (
            ' --cluster-id j-ABCD --dir s3://abc/  --type incremental'
            + ' --interval 10 --unit minutes'
        )
        cmdline = self.prefix + args

        steps = deepcopy(self.default_steps)
        args = steps[0]['HadoopJarStep']['Args']
        args[5] = '--incremental-backup-time-interval'
        args[7] = '--incremental-backup-time-unit'
        steps[0]['HadoopJarStep']['Args'] = args

        result = {'JobFlowId': 'j-ABCD', 'Steps': steps}

        self.assert_params_for_cmd(cmdline, result)

    def test_schedule_hbase_backup_wrong_type(self):
        args = (
            ' --cluster-id j-ABCD --dir s3://abc/  --type wrong_type'
            + ' --interval 10 --unit minutes'
        )
        cmdline = self.prefix + args
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): invalid type. type should be'
            + ' either full or incremental.\n'
        )
        result = self.run_cmd(cmdline, 252)

        self.assertEqual(expected_error_msg, result[1])

    def test_schedule_hbase_backup_wrong_unit(self):
        args = (
            ' --cluster-id j-ABCD --dir s3://abc/  --type full'
            + ' --interval 10 --unit wrong_unit'
        )
        cmdline = self.prefix + args
        expected_error_msg = (
            '\naws: [ERROR]: An error occurred (ParamValidation): invalid unit. unit should be'
            + ' one of the following values: minutes,'
            + ' hours or days.\n'
        )
        result = self.run_cmd(cmdline, 252)

        self.assertEqual(expected_error_msg, result[1])

    def test_schedule_hbase_backup_consistent(self):
        args = (
            ' --cluster-id j-ABCD --dir s3://abc/ --type full'
            + ' --interval 10 --unit minutes --consistent'
        )
        cmdline = self.prefix + args

        steps = deepcopy(self.default_steps)
        steps[0]['HadoopJarStep']['Args'].insert(5, '--consistent')

        result = {'JobFlowId': 'j-ABCD', 'Steps': steps}
        self.assert_params_for_cmd(cmdline, result)

    def test_schedule_hbase_backup_start_time(self):
        args = (
            ' --cluster-id j-ABCD --dir s3://abc/ --type full --interval'
            + ' 10 --unit minutes --start-time 2014-04-18T10:43:24-07:00'
        )
        cmdline = self.prefix + args

        steps = deepcopy(self.default_steps)
        steps[0]['HadoopJarStep']['Args'][10] = '2014-04-18T10:43:24-07:00'

        result = {'JobFlowId': 'j-ABCD', 'Steps': steps}
        self.assert_params_for_cmd(cmdline, result)

    @mock.patch('awscli.customizations.emr.' 'emrutils.get_release_label')
    def test_unsupported_command_on_release_based_cluster_error(
        self, grl_patch
    ):
        grl_patch.return_value = 'emr-4.0'
        args = (
            ' --cluster-id j-ABCD --dir s3://abc/ --type full'
            + ' --interval 10 --unit minutes'
        )
        cmdline = self.prefix + args
        expected_error_msg = (
            "\naws: [ERROR]: An error occurred (ParamValidation): schedule-hbase-backup"
            " is not supported with 'emr-4.0' release.\n"
        )
        result = self.run_cmd(cmdline, 252)

        self.assertEqual(result[1], expected_error_msg)


if __name__ == "__main__":
    unittest.main()



# Copyright 2020 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import errno
import json

import awscli
from awscli.testutils import (
    BaseAWSCommandParamsTest,
    BaseAWSHelpOutputTest,
    mock,
)


class TestPushContainerImageTest(BaseAWSCommandParamsTest):
    def get_push_container_command(self, extra_params):
        cmd = (
            'lightsail '
            'push-container-image '
            '--service-name=dyservicev3 '
            '--label david16  '
            '--image hello:latest '
        )
        if extra_params is not None:
            cmd += extra_params
        return cmd

    def assert_expected_input_options(self, mock_subprocess_run):
        self.assertEqual(
            ['lightsailctl', '--plugin', '--input-stdin'],
            mock_subprocess_run.call_args[0][0],
        )

    def assert_expected_input_version(self, actual_input_version):
        self.assertEqual('1', actual_input_version)

    def assert_expected_operation(self, actual_operation):
        self.assertEqual('PushContainerImage', actual_operation)

    def assert_expected_payload(self, actual_payload):
        self.assertEqual(
            {
                'service': 'dyservicev3',
                'image': 'hello:latest',
                'label': 'david16',
            },
            actual_payload,
        )

    def assert_expected_configuration(
        self, configuration_payload, **expected_options
    ):
        for option, value in expected_options.items():
            self.assertEqual(configuration_payload[option], value)

    @mock.patch(
        'awscli.customizations.lightsail.'
        'push_container_image.subprocess.run'
    )
    def test_start_lightsailctl_failed(self, mock_subprocess_run):
        cmdline = (
            'lightsail '
            '--region us-west-2 '
            '--endpoint-url https://override.custom.endpoint.com '
            'push-container-image '
            '--service-name=dyservicev3 '
            '--label david16  '
        )
        mock_subprocess_run.side_effect = OSError(errno.ENOENT, 'some error')
        response = self.run_cmd(cmdline, expected_rc=252)
        self.assertEqual(response[2], 252)

    @mock.patch(
        'awscli.customizations.lightsail.'
        'push_container_image.subprocess.run'
    )
    def test_start_lightsailctl_success(self, mock_subprocess_run):
        cmdline = self.get_push_container_command(None)

        mock_subprocess_run.return_value = 0
        response = self.run_cmd(cmdline)

        self.assert_expected_input_options(mock_subprocess_run)

        actual_input = json.loads(mock_subprocess_run.call_args[1]['input'])
        self.assert_expected_input_version(actual_input['inputVersion'])
        self.assert_expected_operation(actual_input['operation'])
        self.assert_expected_payload(actual_input['payload'])
        self.assert_expected_configuration(
            actual_input['configuration'],
            debug=False,
            paginate=True,
            region='us-east-1',
            doNotSignRequest=False,
            readTimeout=60,
            connectTimeout=60,
            cliVersion=awscli.__version__,
        )
        self.assertEqual(response[2], 0)

    @mock.patch(
        'awscli.customizations.lightsail.'
        'push_container_image.subprocess.run'
    )
    def test_start_lightsailctl_with_region_success(self, mock_subprocess_run):
        cmdline = self.get_push_container_command('--region us-west-2')

        mock_subprocess_run.return_value = 0
        response = self.run_cmd(cmdline)

        self.assert_expected_input_options(mock_subprocess_run)

        actual_input = json.loads(mock_subprocess_run.call_args[1]['input'])
        self.assert_expected_input_version(actual_input['inputVersion'])
        self.assert_expected_operation(actual_input['operation'])
        self.assert_expected_payload(actual_input['payload'])
        self.assert_expected_configuration(
            actual_input['configuration'], region='us-west-2'
        )
        self.assertEqual(response[2], 0)

    @mock.patch(
        'awscli.customizations.lightsail.'
        'push_container_image.subprocess.run'
    )
    def test_start_lightsailctl_with_url_success(self, mock_subprocess_run):
        extra_param = '--endpoint-url https://test.amazon.com '
        cmdline = self.get_push_container_command(extra_param)

        mock_subprocess_run.return_value = 0
        response = self.run_cmd(cmdline)

        self.assert_expected_input_options(mock_subprocess_run)

        actual_input = json.loads(mock_subprocess_run.call_args[1]['input'])
        self.assert_expected_input_version(actual_input['inputVersion'])
        self.assert_expected_operation(actual_input['operation'])
        self.assert_expected_payload(actual_input['payload'])
        self.assert_expected_configuration(
            actual_input['configuration'], endpoint='https://test.amazon.com'
        )
        self.assertEqual(response[2], 0)

    @mock.patch(
        'awscli.customizations.lightsail.'
        'push_container_image.subprocess.run'
    )
    def test_start_lightsailctl_with_no_verify_ssl_ssuccess(
        self, mock_subprocess_run
    ):
        extra_param = '--no-verify-ssl '
        cmdline = self.get_push_container_command(extra_param)

        mock_subprocess_run.return_value = 0
        response = self.run_cmd(cmdline)

        self.assert_expected_input_options(mock_subprocess_run)

        actual_input = json.loads(mock_subprocess_run.call_args[1]['input'])
        self.assert_expected_input_version(actual_input['inputVersion'])
        self.assert_expected_operation(actual_input['operation'])
        self.assert_expected_payload(actual_input['payload'])
        self.assert_expected_configuration(
            actual_input['configuration'], doNotVerifySSL=True
        )
        self.assertEqual(response[2], 0)

    @mock.patch(
        'awscli.customizations.lightsail.'
        'push_container_image.subprocess.run'
    )
    def test_start_lightsailctl_with_no_sign_request_ssuccess(
        self, mock_subprocess_run
    ):
        extra_param = '--no-sign-request '
        cmdline = self.get_push_container_command(extra_param)

        mock_subprocess_run.return_value = 0
        response = self.run_cmd(cmdline)

        self.assert_expected_input_options(mock_subprocess_run)

        actual_input = json.loads(mock_subprocess_run.call_args[1]['input'])
        self.assert_expected_input_version(actual_input['inputVersion'])
        self.assert_expected_operation(actual_input['operation'])
        self.assert_expected_payload(actual_input['payload'])
        self.assert_expected_configuration(
            actual_input['configuration'], doNotSignRequest=True
        )
        self.assertEqual(response[2], 0)

    @mock.patch(
        'awscli.customizations.lightsail.'
        'push_container_image.subprocess.run'
    )
    def test_start_lightsailctl_with_no_paginate_ssuccess(
        self, mock_subprocess_run
    ):
        extra_param = '--no-paginate '
        cmdline = self.get_push_container_command(extra_param)

        mock_subprocess_run.return_value = 0
        response = self.run_cmd(cmdline)

        self.assert_expected_input_options(mock_subprocess_run)

        actual_input = json.loads(mock_subprocess_run.call_args[1]['input'])
        self.assert_expected_input_version(actual_input['inputVersion'])
        self.assert_expected_operation(actual_input['operation'])
        self.assert_expected_payload(actual_input['payload'])
        self.assert_expected_configuration(
            actual_input['configuration'], paginate=False
        )
        self.assertEqual(response[2], 0)

    @mock.patch(
        'awscli.customizations.lightsail.'
        'push_container_image.subprocess.run'
    )
    def test_start_lightsailctl_with_output_json_ssuccess(
        self, mock_subprocess_run
    ):
        extra_param = '--output json'
        cmdline = self.get_push_container_command(extra_param)

        mock_subprocess_run.return_value = 0
        response = self.run_cmd(cmdline)

        self.assert_expected_input_options(mock_subprocess_run)

        actual_input = json.loads(mock_subprocess_run.call_args[1]['input'])
        self.assert_expected_input_version(actual_input['inputVersion'])
        self.assert_expected_operation(actual_input['operation'])
        self.assert_expected_payload(actual_input['payload'])
        self.assert_expected_configuration(
            actual_input['configuration'], output='json'
        )
        self.assertEqual(response[2], 0)

    @mock.patch(
        'awscli.customizations.lightsail.'
        'push_container_image.subprocess.run'
    )
    def test_start_lightsailctl_with_read_timeout_ssuccess(
        self, mock_subprocess_run
    ):
        extra_param = '--cli-read-timeout 70 '
        cmdline = self.get_push_container_command(extra_param)

        mock_subprocess_run.return_value = 0
        response = self.run_cmd(cmdline)

        self.assert_expected_input_options(mock_subprocess_run)

        actual_input = json.loads(mock_subprocess_run.call_args[1]['input'])
        self.assert_expected_input_version(actual_input['inputVersion'])
        self.assert_expected_operation(actual_input['operation'])
        self.assert_expected_payload(actual_input['payload'])
        self.assert_expected_configuration(
            actual_input['configuration'], readTimeout=70
        )
        self.assertEqual(response[2], 0)

    @mock.patch(
        'awscli.customizations.lightsail.'
        'push_container_image.subprocess.run'
    )
    def test_start_lightsailctl_with_con_timeout_ssuccess(
        self, mock_subprocess_run
    ):
        extra_param = '--cli-connect-timeout 80 '
        cmdline = self.get_push_container_command(extra_param)

        mock_subprocess_run.return_value = 0
        response = self.run_cmd(cmdline)

        self.assert_expected_input_options(mock_subprocess_run)

        actual_input = json.loads(mock_subprocess_run.call_args[1]['input'])
        self.assert_expected_input_version(actual_input['inputVersion'])
        self.assert_expected_operation(actual_input['operation'])
        self.assert_expected_payload(actual_input['payload'])
        self.assert_expected_configuration(
            actual_input['configuration'], connectTimeout=80
        )
        self.assertEqual(response[2], 0)

    @mock.patch(
        'awscli.customizations.lightsail.'
        'push_container_image.subprocess.run'
    )
    def test_start_lightsailctl_with_cabundle_ssuccess(
        self, mock_subprocess_run
    ):
        extra_param = '--ca-bundle testPEM'
        cmdline = self.get_push_container_command(extra_param)

        mock_subprocess_run.return_value = 0
        response = self.run_cmd(cmdline)

        self.assert_expected_input_options(mock_subprocess_run)

        actual_input = json.loads(mock_subprocess_run.call_args[1]['input'])
        self.assert_expected_input_version(actual_input['inputVersion'])
        self.assert_expected_operation(actual_input['operation'])
        self.assert_expected_payload(actual_input['payload'])
        self.assert_expected_configuration(
            actual_input['configuration'], caBundle='testPEM'
        )
        self.assertEqual(response[2], 0)


class TestHelpOutput(BaseAWSHelpOutputTest):
    def test_lightsailctl_output(self):
        self.driver.main(['lightsail', 'push-container-image', 'help'])
        self.assert_contains('push-container-image')



# Copyright 2012-2017 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.

from argparse import Namespace

from botocore.compat import json

from awscli.customizations.servicecatalog import exceptions
from awscli.customizations.servicecatalog.generateproduct import (
    GenerateProductCommand,
)
from awscli.testutils import capture_output, mock, unittest


class TestCreateProductCommand(unittest.TestCase):
    def setUp(self):
        self.session = mock.Mock()
        self.servicecatalog_client = mock.Mock()
        self.s3_client = mock.Mock()
        self.session.create_client.side_effect = [
            self.s3_client,
            self.servicecatalog_client,
        ]
        self.session.get_available_regions.return_value = [
            'us-east-1',
            'eu-west-1',
        ]
        self.cmd = GenerateProductCommand(self.session)

        self.args = Namespace()
        self.args.file_path = 'foo'
        self.args.bucket_name = 'bucket-name'
        self.args.product_name = 'created product name'
        self.args.tags = [
            "Key=key1,Value=value1",
            "Key=key2,Value=value2",
            "Key=key3,Value=value3",
        ]
        self.args.product_owner = 'me'
        self.args.product_type = 'CLOUD_FORMATION_TEMPLATE'
        self.args.product_distributor = 'prod_distributor'
        self.args.provisioning_artifact_name = 'pa-name'
        self.args.provisioning_artifact_description = 'pa-desc'
        self.args.provisioning_artifact_type = 'CLOUD_FORMATION_TEMPLATE'
        self.args.idempotency_token = '123456789'
        self.args.product_description = 'Short description'
        self.args.support_url = 'https://wiki.example.com/CLI/support'
        self.args.support_email = 'cli@example.com'
        self.args.support_description = 'support description'

        self.s3_url = 'https://s3.amazonaws.com/bucket-name/foo'

        # set global args
        self.global_args = Namespace()
        self.global_args.region = 'us-east-1'
        self.global_args.endpoint_url = None
        self.global_args.verify_ssl = None

    @mock.patch('os.path.getsize', return_value=1)
    def test_happy_path(self, getsize_patch):
        # Arrange
        actual_product_view_detail = self.get_product_view_detail()
        self.servicecatalog_client.create_product.return_value = (
            actual_product_view_detail
        )
        expected_product_view_detail = self.get_product_view_detail()
        del expected_product_view_detail["ResponseMetadata"]
        expected_response_output = json.dumps(
            expected_product_view_detail, indent=2
        )
        expected_args = self.get_args_dict()

        # Act
        with capture_output() as captured:
            result = self.cmd._run_main(self.args, self.global_args)

        # Assert
        self.session.create_client.assert_called_with(
            'servicecatalog',
            region_name=self.global_args.region,
            endpoint_url=None,
            verify=None,
        )

        self.servicecatalog_client.create_product.assert_called_once_with(
            Name=expected_args['product-name'],
            Owner=expected_args['product-owner'],
            Description=expected_args['product-description'],
            Distributor=expected_args['product-distributor'],
            SupportDescription=expected_args['support-description'],
            SupportEmail=expected_args['support-email'],
            ProductType=expected_args['product-type'],
            Tags=expected_args['tags'],
            ProvisioningArtifactParameters=self.get_provisioning_artifact_parameters(
                self.args.provisioning_artifact_name,
                self.args.provisioning_artifact_description,
                self.args.provisioning_artifact_type,
            ),
        )
        self.assertEqual(expected_response_output, captured.stdout.getvalue())
        self.assertEqual(0, result)

    @mock.patch('os.path.getsize', return_value=1)
    def test_happy_path_unicode(self, getsize_patch):
        # Arrange
        self.args.product_name = '\u05d1\u05e8\u05d9\u05e6\u05e7\u05dc\u05d4'
        self.args.support_description = '\u00fd\u00a9\u0194\u0292'

        actual_product_view_detail = self.get_product_view_detail()
        self.servicecatalog_client.create_product.return_value = (
            actual_product_view_detail
        )
        expected_product_view_detail = self.get_product_view_detail()
        del expected_product_view_detail["ResponseMetadata"]
        expected_response_output = json.dumps(
            expected_product_view_detail, indent=2
        )
        expected_args = self.get_args_dict()

        # Act
        with capture_output() as captured:
            result = self.cmd._run_main(self.args, self.global_args)

        # Assert
        self.session.create_client.assert_called_with(
            'servicecatalog',
            region_name=self.global_args.region,
            endpoint_url=None,
            verify=None,
        )

        self.servicecatalog_client.create_product.assert_called_once_with(
            Name=expected_args['product-name'],
            Owner=expected_args['product-owner'],
            Description=expected_args['product-description'],
            Distributor=expected_args['product-distributor'],
            SupportDescription=expected_args['support-description'],
            SupportEmail=expected_args['support-email'],
            ProductType=expected_args['product-type'],
            Tags=expected_args['tags'],
            ProvisioningArtifactParameters=self.get_provisioning_artifact_parameters(
                self.args.provisioning_artifact_name,
                self.args.provisioning_artifact_description,
                self.args.provisioning_artifact_type,
            ),
        )
        self.assertEqual(expected_response_output, captured.stdout.getvalue())
        self.assertEqual(0, result)

    def test_region_not_supported(self):
        self.global_args.region = 'not-supported-region'
        with self.assertRaisesRegex(
            exceptions.InvalidParametersException, "not supported"
        ):
            self.cmd._run_main(self.args, self.global_args)

    @mock.patch('os.path.getsize', return_value=1)
    def test_happy_path_omitting_optional_parameters(self, getsize_patch):
        # Arrange
        self.args.support_description = None
        self.args.product_description = None
        self.args.support_email = None
        self.args.product_distributor = None

        actual_product_view_detail = self.get_product_view_detail()
        self.servicecatalog_client.create_product.return_value = (
            actual_product_view_detail
        )
        expected_product_view_detail = self.get_product_view_detail()
        del expected_product_view_detail["ResponseMetadata"]
        expected_response_output = json.dumps(
            expected_product_view_detail, indent=2
        )
        expected_args = self.get_args_dict()

        # Act
        with capture_output() as captured:
            result = self.cmd._run_main(self.args, self.global_args)

        # Assert
        self.session.create_client.assert_called_with(
            'servicecatalog',
            region_name=self.global_args.region,
            endpoint_url=None,
            verify=None,
        )
        self.servicecatalog_client.create_product.assert_called_once_with(
            Name=expected_args['product-name'],
            Owner=expected_args['product-owner'],
            ProductType=expected_args['product-type'],
            Tags=expected_args['tags'],
            ProvisioningArtifactParameters=self.get_provisioning_artifact_parameters(
                self.args.provisioning_artifact_name,
                self.args.provisioning_artifact_description,
                self.args.provisioning_artifact_type,
            ),
        )

        self.assertEqual(expected_response_output, captured.stdout.getvalue())
        self.assertEqual(0, result)

    def get_product_view_detail(self):
        return {
            'ProductViewDetail': {
                'ProductViewSummary': {
                    'SupportDescription': 'Contact the CLI department '
                    'for issues '
                    'deploying or connecting '
                    'to this product.',
                    'HasDefaultPath': False,
                    'ShortDescription': self.args.product_description,
                    'SupportUrl': self.args.support_url,
                    'Distributor': '',
                    'SupportEmail': self.args.support_email,
                    'Type': self.args.product_type,
                    'Id': 'prodview-ask5ma4y7gwiy',
                    'ProductId': 'prod-inifmhpr47ft2',
                },
                'Status': 'CREATED',
                'ProductARN': 'arn:aws:catalog:us-west-2:'
                '856570181934:product/prod-inifmhpr47ft2',
                'CreatedTime': 1492728697.0,
            },
            "ProvisioningArtifactDetail": {
                "CreatedTime": 1502495200.0,
                "Description": "pa_desc",
                "Type": "CLOUD_FORMATION_TEMPLATE",
                "Id": "pa-6hgqvd3pnwbgc",
                "Name": "pa_name",
            },
            "Tags": self.get_tags_dictionary(),
            "ResponseMetadata": {},
        }

    def get_args_dict(self):
        return {
            'file-path': self.args.file_path,
            'bucket-name': self.args.bucket_name,
            'product-name': self.args.product_name,
            'tags': self.get_tags_dictionary(),
            'product-owner': self.args.product_owner,
            'product-type': self.args.product_type,
            'product-description': self.args.product_description,
            'product-distributor': self.args.product_distributor,
            'support-description': self.args.support_description,
            'support-email': self.args.support_email,
        }

    def get_tags_dictionary(self):
        return [
            {"Value": "value1", "Key": "key1"},
            {"Value": "value2", "Key": "key2"},
            {"Value": "value3", "Key": "key3"},
        ]

    def get_provisioning_artifact_parameters(
        self, pa_name, pa_description, pa_type
    ):
        parameters = {
            'Name': pa_name,
            'Description': pa_description,
            'Info': {'LoadTemplateFromURL': self.s3_url},
            'Type': pa_type,
        }

        return parameters



# Copyright 2022 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
#     http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os
import subprocess

import pytest

from tests.backends.build_system.integration import (
    BaseArtifactTest,
    VEnvWorkspace,
)


class TestBuildBackend(BaseArtifactTest):
    def test_exe_with_deps(self, exe_deps: VEnvWorkspace):
        self.assert_built_exe_is_correct(exe_deps.cli_path)

    def test_exe_without_deps(self, exe_no_deps: VEnvWorkspace):
        self.assert_built_exe_is_correct(exe_no_deps.cli_path)

    def test_venv_with_deps(self, sdist_deps: VEnvWorkspace):
        self.assert_built_venv_is_correct(sdist_deps.build_path)

    def test_venv_without_deps(self, sdist_no_deps: VEnvWorkspace):
        self.assert_built_venv_is_correct(sdist_no_deps.build_path)


class TestBuildBackendFailureCases:
    def test_errors_building_exe_without_pyinstaller(self, workspace, capsys):
        workspace.install_dependencies()
        with pytest.raises(subprocess.CalledProcessError) as e:
            workspace.call_build_system("portable-exe", download_deps=False)
        error_text = e.value.stdout.decode()
        assert "pyinstaller" in error_text
        assert "No such file or directory" in error_text

    def test_errors_building_venv_without_runtime_deps(self, workspace):
        with pytest.raises(subprocess.CalledProcessError) as e:
            workspace.call_build_system("system-sandbox", download_deps=False)
        error_text = e.value.stdout.decode()
        possible_messages = [
            "No module named 'flit_core'",
            # Recent versions of pyproject-hooks started consuming
            # ImportError messages and replacing it.
            "Cannot import 'pep517'",
        ]
        assert any([msg in error_text for msg in possible_messages])


class TestInstall(BaseArtifactTest):
    def test_install_exe(self, exe_deps: VEnvWorkspace):
        exe_deps.call_install(
            bin_path=exe_deps.bin_path,
            lib_path=exe_deps.lib_path,
        )

        self.assert_installed_exe_is_correct(exe_deps.bin_path)

    def test_install_venv(self, sdist_deps: VEnvWorkspace):
        sdist_deps.call_install(
            bin_path=sdist_deps.bin_path,
            lib_path=sdist_deps.lib_path,
        )

        self.assert_installed_venv_is_correct(
            sdist_deps.bin_path,
            sdist_deps.lib_path,
        )


class TestUninstall(BaseArtifactTest):
    def test_uninstall_exe(self, exe_deps: VEnvWorkspace):
        exe_deps.call_install(
            bin_path=exe_deps.bin_path,
            lib_path=exe_deps.lib_path,
        )

        exe_deps.call_uninstall(
            bin_path=exe_deps.bin_path,
            lib_path=exe_deps.lib_path,
        )

        assert os.listdir(exe_deps.bin_path) == []
        assert os.listdir(exe_deps.lib_path) == []

    def test_uninstall_venv(self, sdist_deps: VEnvWorkspace):
        sdist_deps.call_install(
            bin_path=sdist_deps.bin_path,
            lib_path=sdist_deps.lib_path,
        )

        sdist_deps.call_uninstall(
            bin_path=sdist_deps.bin_path,
            lib_path=sdist_deps.lib_path,
        )

        assert os.listdir(sdist_deps.bin_path) == []
        assert os.listdir(sdist_deps.lib_path) == []



# Copyright 2015 Amazon.com, Inc. or its affiliates. All Rights Reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License"). You
# may not use this file except in compliance with the License. A copy of
# the License is located at
#
# http://aws.amazon.com/apache2.0/
#
# or in the "license" file accompanying this file. This file is
# distributed on an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF
# ANY KIND, either express or implied. See the License for the specific
# language governing permissions and limitations under the License.
import os

from awscli.testutils import BaseAWSCommandParamsTest, FileCreator, mock


class TestUploadBuild(BaseAWSCommandParamsTest):
    prefix = 'gamelift upload-build'

    def setUp(self):
        super(TestUploadBuild, self).setUp()
        self.files = FileCreator()

    def tearDown(self):
        super(TestUploadBuild, self).tearDown()
        self.files.remove_all()

    def test_upload_build(self):
        self.files.create_file('tmpfile', 'Some contents')
        cmdline = self.prefix
        cmdline += ' --name mybuild --build-version myversion'
        cmdline += ' --build-root %s' % self.files.rootdir

        self.parsed_responses = [
            {'Build': {'BuildId': 'myid'}},
            {
                'StorageLocation': {'Bucket': 'mybucket', 'Key': 'mykey'},
                'UploadCredentials': {
                    'AccessKeyId': 'myaccesskey',
                    'SecretAccessKey': 'mysecretkey',
                    'SessionToken': 'mytoken',
                },
            },
            {},
        ]

        stdout, stderr, rc = self.run_cmd(cmdline, expected_rc=0)

        # First the build is created.
        self.assertEqual(len(self.operations_called), 3)
        self.assertEqual(self.operations_called[0][0].name, 'CreateBuild')
        self.assertEqual(
            self.operations_called[0][1],
            {'Name': 'mybuild', 'Version': 'myversion'},
        )

        # Second the credentials are requested.
        self.assertEqual(
            self.operations_called[1][0].name, 'RequestUploadCredentials'
        )
        self.assertEqual(self.operations_called[1][1], {'BuildId': 'myid'})

        # The build is then uploaded to S3.
        self.assertEqual(self.operations_called[2][0].name, 'PutObject')
        self.assertEqual(
            self.operations_called[2][1],
            {'Body': mock.ANY, 'Bucket': 'mybucket', 'Key': 'mykey'},
        )

        # Check the output of the command.
        self.assertIn(
            'Successfully uploaded %s to AWS GameLift' % self.files.rootdir,
            stdout,
        )
        self.assertIn('Build ID: myid', stdout)

    def test_upload_build_with_operating_system_param(self):
        self.files.create_file('tmpfile', 'Some contents')
        cmdline = self.prefix
        cmdline += ' --name mybuild --build-version myversion'
        cmdline += ' --build-root %s' % self.files.rootdir
        cmdline += ' --operating-system WINDOWS_2012'

        self.parsed_responses = [
            {'Build': {'BuildId': 'myid'}},
            {
                'StorageLocation': {'Bucket': 'mybucket', 'Key': 'mykey'},
                'UploadCredentials': {
                    'AccessKeyId': 'myaccesskey',
                    'SecretAccessKey': 'mysecretkey',
                    'SessionToken': 'mytoken',
                },
            },
            {},
        ]

        stdout, stderr, rc = self.run_cmd(cmdline, expected_rc=0)

        # First the build is created.
        self.assertEqual(len(self.operations_called), 3)
        self.assertEqual(self.operations_called[0][0].name, 'CreateBuild')
        self.assertEqual(
            self.operations_called[0][1],
            {
                'Name': 'mybuild',
                'Version': 'myversion',
                'OperatingSystem': 'WINDOWS_2012',
            },
        )

        # Second the credentials are requested.
        self.assertEqual(
            self.operations_called[1][0].name, 'RequestUploadCredentials'
        )
        self.assertEqual(self.operations_called[1][1], {'BuildId': 'myid'})

        # The build is then uploaded to S3.
        self.assertEqual(self.operations_called[2][0].name, 'PutObject')
        self.assertEqual(
            self.operations_called[2][1],
            {'Body': mock.ANY, 'Bucket': 'mybucket', 'Key': 'mykey'},
        )

        # Check the output of the command.
        self.assertIn(
            'Successfully uploaded %s to AWS GameLift' % self.files.rootdir,
            stdout,
        )
        self.assertIn('Build ID: myid', stdout)

    def test_upload_build_with_empty_directory(self):
        cmdline = self.prefix
        cmdline += ' --name mybuild --build-version myversion'
        cmdline += ' --build-root %s' % self.files.rootdir

        self.parsed_responses = [
            {'Build': {'BuildId': 'myid'}},
            {
                'StorageLocation': {'Bucket': 'mybucket', 'Key': 'mykey'},
                'UploadCredentials': {
                    'AccessKeyId': 'myaccesskey',
                    'SecretAccessKey': 'mysecretkey',
                    'SessionToken': 'mytoken',
                },
            },
            {},
        ]

        stdout, stderr, rc = self.run_cmd(cmdline, expected_rc=255)

        self.assertIn(
            'Fail to upload %s. '
            'The build root directory is empty or does not exist.\n'
            % self.files.rootdir,
            stderr,
        )

    def test_upload_build_with_nonexistent_directory(self):
        dir_not_exist = os.path.join(self.files.rootdir, 'does_not_exist')

        cmdline = self.prefix
        cmdline += ' --name mybuild --build-version myversion'
        cmdline += ' --build-root %s' % dir_not_exist

        self.parsed_responses = [
            {'Build': {'BuildId': 'myid'}},
            {
                'StorageLocation': {'Bucket': 'mybucket', 'Key': 'mykey'},
                'UploadCredentials': {
                    'AccessKeyId': 'myaccesskey',
                    'SecretAccessKey': 'mysecretkey',
                    'SessionToken': 'mytoken',
                },
            },
            {},
        ]

        stdout, stderr, rc = self.run_cmd(cmdline, expected_rc=255)

        self.assertIn(
            'Fail to upload %s. '
            'The build root directory is empty or does not exist.\n'
            % dir_not_exist,
            stderr,
        )

    def test_upload_build_with_nonprovided_directory(self):
        cmdline = self.prefix
        cmdline += ' --name mybuild --build-version myversion'
        cmdline += ' --build-root %s' % '""'

        self.parsed_responses = [
            {'Build': {'BuildId': 'myid'}},
            {
                'StorageLocation': {'Bucket': 'mybucket', 'Key': 'mykey'},
                'UploadCredentials': {
                    'AccessKeyId': 'myaccesskey',
                    'SecretAccessKey': 'mysecretkey',
                    'SessionToken': 'mytoken',
                },
            },
            {},
        ]

        stdout, stderr, rc = self.run_cmd(cmdline, expected_rc=255)

        self.assertIn(
            'Fail to upload %s. '
            'The build root directory is empty or does not exist.\n' % '""',
            stderr,
        )

    def test_upload_build_with_server_sdk_version_param(self):
        self.files.create_file('tmpfile', 'Some contents')
        cmdline = self.prefix
        cmdline += ' --name mybuild --build-version myversion'
        cmdline += ' --build-root %s' % self.files.rootdir
        cmdline += ' --server-sdk-version 4.0.2'

        self.parsed_responses = [
            {'Build': {'BuildId': 'myid'}},
            {
                'StorageLocation': {'Bucket': 'mybucket', 'Key': 'mykey'},
                'UploadCredentials': {
                    'AccessKeyId': 'myaccesskey',
                    'SecretAccessKey': 'mysecretkey',
                    'SessionToken': 'mytoken',
                },
            },
            {},
        ]

        stdout, stderr, rc = self.run_cmd(cmdline, expected_rc=0)

        # First the build is created.
        self.assertEqual(len(self.operations_called), 3)
        self.assertEqual(self.operations_called[0][0].name, 'CreateBuild')
        self.assertEqual(
            self.operations_called[0][1],
            {
                'Name': 'mybuild',
                'Version': 'myversion',
                'ServerSdkVersion': '4.0.2',
            },
        )

        # Second the credentials are requested.
        self.assertEqual(
            self.operations_called[1][0].name, 'RequestUploadCredentials'
        )
        self.assertEqual(self.operations_called[1][1], {'BuildId': 'myid'})

        # The build is then uploaded to S3.
        self.assertEqual(self.operations_called[2][0].name, 'PutObject')
        self.assertEqual(
            self.operations_called[2][1],
            {'Body': mock.ANY, 'Bucket': 'mybucket', 'Key': 'mykey'},
        )

        # Check the output of the command.
        self.assertIn(
            'Successfully uploaded %s to AWS GameLift' % self.files.rootdir,
            stdout,
        )
        self.assertIn('Build ID: myid', stdout)

    def test_upload_build_with_tags_param(self):
        self.files.create_file('tmpfile', 'Some contents')
        
        expected_tags = [
            {'Key': 'Key1', 'Value': 'Value1'},
            {'Key': 'Key2', 'Value': 'Value2'}
        ]
        
        cmdline = self.prefix
        cmdline += ' --name mybuild --build-version myversion'
        cmdline += ' --build-root %s' % self.files.rootdir
        cmdline += ' --tags'
        for tag in expected_tags:
            cmdline += ' %s=%s' % (tag['Key'], tag['Value'])

        self.parsed_responses = [
            {'Build': {'BuildId': 'myid'}},
            {'StorageLocation': {
                'Bucket': 'mybucket',
                'Key': 'mykey'},
                'UploadCredentials': {
                    'AccessKeyId': 'myaccesskey',
                    'SecretAccessKey': 'mysecretkey',
                    'SessionToken': 'mytoken'}},
            {}
        ]

        stdout, stderr, rc = self.run_cmd(cmdline, expected_rc=0)

        # First the build is created.
        self.assertEqual(len(self.operations_called), 3)
        self.assertEqual(self.operations_called[0][0].name, 'CreateBuild')
        self.assertEqual(
            self.operations_called[0][1],
            {'Name': 'mybuild', 'Version': 'myversion',
             'Tags': expected_tags}
        )

        # Second the credentials are requested.
        self.assertEqual(
            self.operations_called[1][0].name, 'RequestUploadCredentials')
        self.assertEqual(
            self.operations_called[1][1], {'BuildId': 'myid'})

        # The build is then uploaded to S3.
        self.assertEqual(self.operations_called[2][0].name, 'PutObject')
        self.assertEqual(
            self.operations_called[2][1],
            {'Body': mock.ANY, 'Bucket': 'mybucket', 'Key': 'mykey'}
        )

        # Check the output of the command.
        self.assertIn(
            'Successfully uploaded %s to AWS GameLift' % self.files.rootdir,
            stdout)
        self.assertIn('Build ID: myid', stdout)
